
from typing import List


def destroy_unimportant_tags_inplace(soup_tag, tags_to_remove: List[str]):
    """Remove tags like <bold> or <italic>"""
    for tag_to_remove in tags_to_remove:
        for match in soup_tag.find_all(tag_to_remove):
            match.replaceWithChildren()


def create_new_parent_tag(soup_tag, parent_tag_name: str, soup):
    """Wraps soup tag with another parent tag"""
    new_parent_tag = soup.new_tag(parent_tag_name)
    contents = soup_tag.replace_with(new_parent_tag)
    new_parent_tag.append(contents)
    return new_parent_tag






class TrainingArguments():

    def __init__(self):
        original_work_dir: ${hydra: runtime.cwd}
        data_dir: ${original_work_dir} / data /
        print_config: true
        ignore_warnings: true
        train: true
        test: true
        seed: null
        name: default
        model_name: scibert - uncased.pt
        datamodule:
        _target_: SciAssist.datamodules.mup_datamodule.MupDataModule
        data_repo: allenai / mup
        train_batch_size: 4
        num_workers: 0
        pin_memory: false
        data_cache_dir: ${paths.data_dir} / mup /
        data_utils:
        _target_: SciAssist.utils.data_utils.DataUtilsForSeq2Seq


        model:
        _target_: SciAssist.models.cora_module.CoraLitModule
        lr: 2.0e-05
        data_utils:
        _target_: SciAssist.utils.data_utils.DataUtilsForTokenClassification
        model:
        _target_: SciAssist.models.components.bert_token_classifier.BertForTokenClassifier
        model_checkpoint: allenai / scibert_scivocab_uncased
        output_size: 13
        cache_dir: ${paths.root_dir} /.cache /
        save_name: ${model_name}
        model_dir: ${paths.model_dir}
        callbacks:
        model_checkpoint:
        _target_: pytorch_lightning.callbacks.ModelCheckpoint
        monitor: val / micro_f1
        mode: max
        save_top_k: ${trainer.max_epochs}
        save_last: true
        verbose: false
        dirpath: ${paths.ckpt_dir}
        filename: epoch_
        {epoch: 03d}
        auto_insert_metric_name: false
        early_stopping:
        _target_: pytorch_lightning.callbacks.EarlyStopping
        monitor: val / micro_f1
        mode: max
        patience: 100
        min_delta: 0
        model_summary:
        _target_: pytorch_lightning.callbacks.RichModelSummary
        max_depth: -1
        rich_progress_bar:
        _target_: pytorch_lightning.callbacks.RichProgressBar
        logger:
        wandb:
        _target_: pytorch_lightning.loggers.wandb.WandbLogger
        project: ce384ad053376bf4d5e6ab3ba097719c5a4ed579
        save_dir: ${paths.log_dir}
        offline: false
        id: null
        log_model: false
        prefix: ''
        job_type: train
        group: ''
        tags: []
        trainer:
        _target_: pytorch_lightning.Trainer
        accelerator: gpu
        devices: 1
        min_epochs: 1
        max_epochs: 5
        resume_from_checkpoint: null
        paths:
        root_dir: ${oc.env: PROJECT_ROOT}
        data_dir: ${paths.root_dir} / data /
        log_dir: ${paths.root_dir} / logs /
        output_dir: ${paths.root_dir} / output
        work_dir: ${hydra: runtime.cwd}
        model_dir: ${paths.root_dir} / pretrained /${name}
        ckpt_dir: ${paths.model_dir} / checkpoints



import csv

from datasets import Dataset


def csv_reader(filename: str, encoding = 'ISO-8859-1'):
    """

    Args:
        filename (`str`): The name of the input CSV file

    Returns:
        data_set (`Dataset`):
            A dataset, for example:
                Dataset({
                    features: ['', 'paper_name', 'text', 'summary', 'paper_id',...],
                    num_rows: 4611
                })

    """
    data_set = {}
    with open(filename, 'r', newline='', encoding=encoding) as f:
        rows = csv.reader(f)
        # Get Column names
        keys = next(rows)
        for key in keys:
            data_set[key] = []
        # Add values by column
        for row in rows:
            for id, key in enumerate(keys):
                data_set[key].append(row[id])

    # Remove columns that aren't involved in the input of model
    # in case of ValueError: Unable to create tensor,
    # you should probably activate truncation and/or
    # padding with 'padding=True' 'truncation=True'
    # to have batched tensors with the same length.
    if "" in data_set.keys():
        data_set.pop("")
    data_set = Dataset.from_dict(data_set)
    return data_set



from transformers import *
from torch.nn.utils.rnn import pad_sequence
import torch.nn as nn
from torch.nn import CrossEntropyLoss
from torchcrf import CRF
import torch

class BertForDatasetExtraction(nn.Module):
    def __init__(
        self,
        model_checkpoint: str = "allenai/scibert_scivocab_uncased",
        cache_dir: str = ".cache",
        save_name: str = "scibert-ner.pt",
        model_dir: str = "pretrained"
    ):
        super().__init__()
        self.save_name = save_name
        self.model_dir = model_dir
        self.num_ner_tags = 3
        self.num_cls_labels = 2
        self.bert = AutoModel.from_pretrained(model_checkpoint, cache_dir=cache_dir)

        self.dropout = nn.Dropout(self.bert.config.hidden_dropout_prob)
        self.crf = CRF(self.num_ner_tags, batch_first=True)
        self.cel = CrossEntropyLoss()
        self.ner_classifier = nn.Linear(self.bert.config.hidden_size, self.num_ner_tags)
        self.cls_classifier = nn.Linear(self.bert.config.hidden_size, self.num_cls_labels)
        # self.init_weights()

    def forward(self, input_subwords, input_token_start_indexs, attention_mask=None, ner_tags=None, cls_labels=None):
        outputs = self.bert(input_subwords,
                            attention_mask=attention_mask,
                            token_type_ids=None,
                            position_ids=None,
                            head_mask=None,
                            inputs_embeds=None)
        sequence_output = outputs[0]

        # obtain original token representations from subwords representations (by selecting the first subword)
        origin_sequence_output = [layer[starts.nonzero().squeeze(1)] for layer, starts in zip(sequence_output, input_token_start_indexs)]
        padded_sequence_output = pad_sequence(origin_sequence_output, batch_first=True)
        padded_sequence_output = self.dropout(padded_sequence_output)

        ner_logits = self.ner_classifier(padded_sequence_output)

        cls_logits = self.cls_classifier(sequence_output[:, 0, :]) # 分类任务
        # cls_logits = self.cls_classifier(outputs[1]) # 分类任务

        outputs = (ner_logits, cls_logits,)

        if ner_tags is not None:
            alpha, beta = 1, 0 # alpha(0~1)控制两个任务的损失权重， beta控制额外引入的正则项
            mask = ner_tags.gt(-1)

            loss_ner = self.crf(ner_logits, ner_tags, mask) * (-1)
            loss_cls = self.cel(cls_logits.view(-1, 2), cls_labels.view(-1))
            loss_penalty = torch.mean((2 * torch.argmax(cls_logits, dim=-1) - 1.0) * torch.mean(ner_logits[:,:,-1], dim=-1))
            loss = alpha * loss_ner + (1- alpha) * loss_cls + beta * loss_penalty

            outputs = (loss,) + outputs

        return outputs




import os
import numpy as np
import requests

url = "https://api.writesonic.com/v2/business/content/chatsonic?engine=premium"


def res(text):

    payload = {
        "enable_google_results": "true",
        "enable_memory": False,
        "input_text": text
    }
    headers = {
        "accept": "application/json",
        "content-type": "application/json",
        "X-API-KEY": "c7d55448-fb45-4d76-af54-2ce50bd01531"
    }

    response = requests.post(url, json=payload, headers=headers)
    return response.text

root_dir = "/home/dingyx/project/SciAssist/data/pdfs/"
for dirpath,dirnames,files in os.walk(root_dir):
    file_l = files
    break
file_l.sort()
file_list=[]
for i in file_l:
    if i[-4:]==".txt":
        file_list.append(i)

mad = []
for i in file_list:
    file_name = os.path.join(root_dir, i)
    input_text = ""
    with open(file_name, "r") as f:
        input_text = f.readlines()
        input_text = [i.strip() for i in input_text]
        input_text = " ".join(input_text)
    for j in range(5):
        length = (j+1)*50
        prompt = f"Give a summary of the following text, which has about {length} words: "
        summ = res(prompt+input_text)
        MAD = abs(len(summ) - length)/50
        mad.append(MAD)

print(np.mean(mad))



import os

from SciAssist.bin.doc2json.doc2json.grobid2json.process_pdf import process_pdf_file

from SciAssist.utils.pdf2text import get_bodytext

for (root,dirs,files) in os.walk("/home/dingyx/project/SciAssist/data/longsumm/pdf"):
    for file in files:
        pdf_file = os.path.join(root,file)
        try:
            json_file = process_pdf_file(input_file=pdf_file, temp_dir="/home/dingyx/project/SciAssist/data/longsumm/temp", output_dir="/home/dingyx/project/SciAssist/data/longsumm/temp")
            path = get_bodytext(json_file=json_file, output_dir = "/home/dingyx/project/SciAssist/data/longsumm/", suffix="")
        except:
            print("Unavailable")



import random

import datasets
import pandas as pd
import os
from tqdm import tqdm
from transformers import AutoModelForSeq2SeqLM, AutoTokenizer
device = "cuda:1"
model = AutoModelForSeq2SeqLM.from_pretrained("google/flan-t5-xl")
tokenizer = AutoTokenizer.from_pretrained("google/flan-t5-xl", max_length=1024, truncation=True)
model.to(device)


file_list = []

summ = []
# root_dir = "/home/yixi/project/sciassist/data/pdfs/text"
root_dir = "/home/yixi/project/scisumm-corpus/data/Training-Set-2019/Task2/From-ScisummNet-2019/"
for dirpath,dirnames,files in os.walk(root_dir):
    file_list = dirnames
    break
# root_dir = "/home/yixi/project/sciassist/data/pdfs/text/"
# for dirpath,dirnames,files in os.walk(root_dir):
#     file_l = files
#     break
# file_l.sort()
# file_list=[]
# for i in file_l:
#     if i[-4:]==".txt":
#         file_list.append(i)


with open("/home/yixi/project/concepts.txt", "r") as f:
    phrases = f.readlines()
    phrases = [i.strip() for i in phrases]
def keyword_extraction(input_text):
    # with open(os.path.join(root_dir,input_text),"r") as f:
    with open(os.path.join(root_dir, input_text, "summary",input_text + ".scisummnet_human.txt"), "r") as f:
        input_text = f.readlines()
        input_text= " ".join(input_text[1:])
    input_text = input_text.replace("\t"," ")
    input_text = input_text.replace("\n","  ")
    keywords = []
    for j in phrases:
        if j in input_text.lower():
            keywords.append(j)
    keywords = list(set(keywords))
    if len(keywords)<1:
        return ""
    keywords = sorted(keywords, key=lambda x:len(x), reverse=True)[:10]
    keywords_reverse=sorted(keywords, key=lambda x:len(x), reverse=False)
    keywords_res = []
    for i in range(len(keywords_reverse)):
        flag=0
        for j in range(i+1,len(keywords)-1):
            if keywords_reverse[i] in keywords_reverse[j]:
                flag=1
                break
        if flag==0:
           keywords_res.append(keywords_reverse[i])

    # print(keywords_reverse)
    keywords_res = [k.strip() for k in keywords_res]
    keywords_res = [k for k in keywords_res if k!=""]


    keyword_str = ",".join(keywords_res)
    # print(keyword_str)


    return keyword_str


# In[197]:


# ### Different setting of keyword extraction

# In[841]:

# raw_datasets = datasets.load_dataset(
#     "allenai/mup",
#     cache_dir="/home/yixi/.cache/sciassist"
# )

print("loading finished.")
# file_list = raw_datasets["train"]
# file_list = raw_datasets["validation"].select(range(1000))

# summ = file_list["summary"]
scisumm = {"File": file_list}
print(file_list)
# print(scisumm["File"])
# scisumm = {"title": file_list["paper_name"], "summary":summ, "text":file_list["text"] }
scisumm = pd.DataFrame(scisumm)
tqdm.pandas(desc='progress bar')
scisumm['keyword']=scisumm["File"].progress_apply(keyword_extraction,args=())
# scisumm['keyword']=scisumm["File"].progress_apply(keyword_extraction,args=())

scisumm.to_csv(os.path.join(root_dir,"forecite.csv"), index=False)


# main developer: Yixi Ding <dingyixi@hotmail.com>

import os
from pathlib import Path
from typing import Any, List

import numpy as np
import torch
from pytorch_lightning import LightningModule
from torchmetrics import MaxMetric
from torchmetrics.text.rouge import ROUGEScore
from SciAssist.models.components.fid_summarization import FiDT5

from SciAssist.utils.data_utils import DataUtilsForFiD


class MupFiDLitModule(LightningModule):
    def __init__(
        self,
        model: FiDT5,
        data_utils = DataUtilsForFiD,
        lr: float = 2e-5,
    ):
        super().__init__()
        self.save_hyperparameters(logger=False)
        self.data_utils = data_utils
        self.model = model

        self.val_metric = ROUGEScore(use_stemmer=True)
        self.val_best_Rouge1 = MaxMetric()
        self.test_metric = ROUGEScore(use_stemmer=True)
        self.test_best_Rouge1 = MaxMetric()
        self.best_Rouge1 = 0

        self.val_gen_lens = []
        self.val_gen_len = 0
        self.test_gen_lens = []
        self.test_gen_len = 0

        self.save_path = Path(os.path.join(self.model.model_dir,self.model.save_name))

    def forward(self, inputs):
        inputs["return_dict"]=False
        return self.hparams.model(**inputs)


    def on_train_start(self):
        self.val_metric.reset()
        self.val_best_Rouge1.reset()
        self.test_metric.reset()
        self.test_best_Rouge1.reset()

    def step(self, batch: Any):
        (labels, _, context_ids, context_mask) = batch
        inputs = {
            "input_ids": context_ids,
            "attention_mask": context_mask,
            "labels": labels
        }
        # print(context_ids.shape)
        outputs = self.forward(inputs)
        loss = outputs[0]
        return loss, labels

    def training_step(self, batch: Any, batch_idx: int):
        outputs = self.step(batch)
        loss = outputs[0]
        self.log("train/loss", loss, on_step=True, on_epoch=True, prog_bar=False)
        return {"loss": loss}

    def training_epoch_end(self, outputs: List[Any]):
        pass

    def validation_step(self, batch: Any, batch_idx: int):
        (labels, _, input_ids, attention_mask) = batch
        outputs = self.step(batch)
        # print("batch",batch)
        # print("outputs",outputs)
        loss = outputs[0]
        self.log("val/loss", loss, on_step=False, on_epoch=True, prog_bar=False)

        # Get prediction ids sequence
        preds = self.model.generate(input_ids=input_ids,attention_mask=attention_mask)

        # Convert ids to strings
        decoded_preds, decoded_labels = self.data_utils.postprocess(preds, labels)

        # Compute Rouge Metrics
        rouge_metric = self.val_metric(preds=decoded_preds, target=decoded_labels)

        result = {key: value * 100 for key, value in rouge_metric.items()}

        # Compute average length of summaries
        self.val_gen_lens.extend([np.count_nonzero(pred != self.data_utils.tokenizer.pad_token_id) for pred in preds.to("cpu")])

        #  Log results
        self.log("val/Rouge-1", result["rouge1_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-1", result["rouge1_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-1", result["rouge1_recall"], on_step=False, on_epoch=True, prog_bar=True)

        self.log("val/Rouge-2", result["rouge2_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-2", result["rouge2_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-2", result["rouge2_recall"], on_step=False, on_epoch=True, prog_bar=True)

        self.log("val/Rouge-L", result["rougeL_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-L", result["rougeL_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-L", result["rougeL_recall"], on_step=False, on_epoch=True, prog_bar=True)

        self.log("val/Rouge-Lsum", result["rougeLsum_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-Lsum", result["rougeLsum_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-Lsum", result["rougeLsum_recall"], on_step=False, on_epoch=True, prog_bar=True)

        return result

    def validation_epoch_end(self, outputs: List[Any]):
        rouge = self.val_metric.compute()
        self.val_best_Rouge1.update(rouge["rouge1_fmeasure"])
        # Save the best model
        if rouge["rouge1_fmeasure"] > self.best_Rouge1:
            self.best_Rouge1 = rouge["rouge1_fmeasure"]
            print(f"Epoch: {self.current_epoch}: Save the current best model.")
            torch.save(self.model.state_dict(), self.save_path)

        # Compute the average length of summaries
        self.val_gen_len = np.mean(self.val_gen_lens)
        self.log("val/gen_len", self.val_gen_len, on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/best_rouge1", self.val_best_Rouge1.compute(), on_epoch=True, prog_bar=True)

    def test_step(self, batch: Any, batch_idx: int):
        (labels, _, input_ids, attention_mask) = batch
        result = {}

        if "labels" is not None:
            # Simply do prediction
            preds = self.model.generate(input_ids=input_ids, attention_mask=attention_mask)
            decoded_preds = self.data_utils.tokenizer.batch_decode(preds, skip_special_tokens=True)
            result["preds"] = decoded_preds

        else:
            # Do prediction and compute metrics
            # Get prediction ids sequence
            preds = self.model.generate(input_ids=input_ids, attention_mask=attention_mask)
            # Convert ids to strings
            decoded_preds, decoded_labels = self.data_utils.postprocess(preds, labels)

            # Compute Rouge Metrics
            rouge_metric = self.test_metric(preds=decoded_preds, target=decoded_labels)

            result = {key: value * 100 for key, value in rouge_metric.items()}
            result['preds'] = decoded_preds
            # Log results
            self.log("test/Rouge-1", result["rouge1_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            self.log("test/Rouge-2", result["rouge2_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            self.log("test/Rouge-L", result["rougeL_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            self.log("test/Rouge-Lsum", result["rougeLsum_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)

        # Compute average length of summaries
        self.test_gen_lens.extend([np.count_nonzero(pred != self.data_utils.tokenizer.pad_token_id) for pred in preds.to("cpu")])
        return result


    def test_epoch_end(self, outputs: List[Any]):
        # Save prediction results
        with open(os.path.join(self.model.model_dir,"prediction.txt"),'w') as f:
            for batch in outputs:
                for res in batch["preds"]:
                    f.write(res)
                    f.write("\n")
        # Compute average length of summaries
        self.test_gen_len = np.mean(self.test_gen_lens)
        self.log("test/gen_len", self.test_gen_len, on_step=False, on_epoch=True, prog_bar=True)



    def on_validation_epoch_end(self):
        self.val_metric.reset()
        self.val_best_Rouge1.reset()
        self.val_gen_lens = []

    def on_test_epoch_end(self):
        self.test_metric.reset()
        self.test_best_Rouge1.reset()
        self.test_gen_lens = []

    def configure_optimizers(self):
        return torch.optim.AdamW(
            params=self.hparams.model.parameters(), lr=self.hparams.lr
        )



import csv
import pandas as pd

ACL = {}
kw = []
text = []
id = []
summ_ours = []
summ_flant5 = []
# scisumm = {"title": file_list["paper_name"], "summary":summ, "text":file_list["text"] }
scisumm = pd.DataFrame(ACL)

with open("/home/dingyx/project/SciAssist/data/pdfs/test.csv", 'r', newline='', encoding='ISO-8859-1') as f:
    rows = csv.reader(f)
    # Get Column names
    keys = next(rows)
    # Add values by column
    for row in rows:
        kws = row[2].split(",")

        kws = kws[0]

        with open("/home/dingyx/project/SciAssist/data/pdfs/summary_ours/" + row[0] + ".txt", "r") as f:
            file = f.readlines()
            file = " ".join(file)
            k = file.strip().split(" => ")[0].strip()
            s_ours = file.strip().split(" => ")[1].strip()
            # s_ours = file.strip()
            summ_ours.append(s_ours)

        with open("/home/dingyx/project/SciAssist/data/pdfs/summary_flant5/" + row[0] + ".txt", "r") as f:
            file = f.readlines()
            file = " ".join(file)
            k = file.strip().split(" => ")[0].strip()
            s_flant5 = file.strip().split(" => ")[1].strip()
            summ_flant5.append(s_flant5)


        kw.append(kws)
        id.append(row[1][:-9])

scisumm["Keyword"] = kw
scisumm["id"] = id
scisumm["System 1"] = summ_ours
scisumm["System 2"] = summ_flant5
scisumm.to_csv("/home/dingyx/project/SciAssist/data/pdfs/human-evaluation-repeat5.csv", index=False)



from nltk.text import TextCollection
from nltk.tokenize import word_tokenize
import datasets
import pandas as pd
import os
from tqdm import tqdm




file_list = []


root_dir = "/home/yixi/project/sciassist/data/pdfs/text"
for dirpath,dirnames,files in os.walk(root_dir):
    file_l = files
    break
file_l.sort()
file_list=[]
for i in file_l:
    if i[-4:]==".txt":
        file_list.append(i)

def keyword_extraction(input_text):
    cnts = {}
    input_text = list(set(input_text))
    for i in input_text:
        tfidf = corpus.tf_idf(i,corpus)
        cnts[i]=tfidf
    cnts = sorted(cnts.items(), key=lambda d: d[1], reverse=True)
    keywords_res = cnts
    print(keywords_res)
    keyword_str = ",".join(keywords_res)
    # print(keyword_str)
    return keyword_str


# In[197]:


# ### Different setting of keyword extraction

# In[841]:

raw_datasets = datasets.load_dataset(
    "allenai/mup",
    cache_dir="/home/yixi/.cache/sciassist"
)

print("loading finished.")
file_list = raw_datasets["train"]


# summ = file_list["summary"]
summ = file_list["text"]
sents=[word_tokenize(sent) for sent in summ]

corpus=TextCollection(sents)

# scisumm = {"File": file_list}
# print(scisumm["File"])
scisumm = {"title": file_list["paper_name"], "summary":file_list["summary"], "text":summ }
scisumm = pd.DataFrame(scisumm)
tqdm.pandas(desc='progress bar')
scisumm['keyword']=scisumm["text"].progress_apply(keyword_extraction,args=())

scisumm.to_csv(os.path.join(root_dir,"test_tfidf.csv"), index=True)


import os

from SciAssist.utils.pdf2text import get_bodytext, get_text_by_section, get_IC

from SciAssist.bin.doc2json.doc2json.grobid2json.process_pdf import process_pdf_file

root_dir = "/home/yixi/project/sciassist/data/pdfs/"
for dirpath,dirnames,files in os.walk(root_dir):
    file_list = files
    break
file_list.sort()
for i in file_list:
    if i[-3:]!="pdf":
        continue
    filename=os.path.join(root_dir,i)

    json_file = process_pdf_file(input_file=filename)
    # Extract bodytext from pdf and save them in TEXT format.
    text_file = get_IC(json_file=json_file, output_dir="/home/yixi/project/sciassist/data/pdfs/text")


import logging
import warnings
from typing import List, Sequence

import pytorch_lightning as pl
import rich.syntax
import rich.tree
from omegaconf import DictConfig, OmegaConf
from pytorch_lightning.utilities import rank_zero_only


def get_logger(name=__name__) -> logging.Logger:
    """Initializes multi-GPU-friendly python command line logger."""

    logger = logging.getLogger(name)

    # this ensures all logging levels get marked with the rank zero decorator
    # otherwise logs would get multiplied for each GPU process in multi-GPU setup
    for level in (
        "debug",
        "info",
        "warning",
        "error",
        "exception",
        "fatal",
        "critical",
    ):
        setattr(logger, level, rank_zero_only(getattr(logger, level)))

    return logger


log = get_logger(__name__)


def extras(config: DictConfig) -> None:
    """Applies optional utilities, controlled by config flags.

    Utilities:
    - Ignoring python warnings
    - Rich config printing
    """

    # disable python warnings if <config.ignore_warnings=True>
    if config.get("ignore_warnings"):
        log.info("Disabling python warnings! <config.ignore_warnings=True>")
        warnings.filterwarnings("ignore")

    # pretty print config tree using Rich library if <config.print_config=True>
    if config.get("print_config"):
        log.info("Printing config tree with Rich! <config.print_config=True>")
        print_config(config, resolve=True)


@rank_zero_only
def print_config(
    config: DictConfig,
    print_order: Sequence[str] = (
        "datamodule",
        "model",
        "callbacks",
        "logger",
        "trainer",
    ),
    resolve: bool = True,
) -> None:
    """Prints content of DictConfig using Rich library and its tree structure.

    Args:
        config (DictConfig): Configuration composed by Hydra.
        print_order (Sequence[str], optional): Determines in what order config components are printed.
        resolve (bool, optional): Whether to resolve reference fields of DictConfig.
    """

    style = "dim"
    tree = rich.tree.Tree("CONFIG", style=style, guide_style=style)

    quee = []

    for field in print_order:
        quee.append(field) if field in config else log.info(f"Field '{field}' not found in config")

    for field in config:
        if field not in quee:
            quee.append(field)

    for field in quee:
        branch = tree.add(field, style=style, guide_style=style)

        config_group = config[field]
        if isinstance(config_group, DictConfig):
            branch_content = OmegaConf.to_yaml(config_group, resolve=resolve)
        else:
            branch_content = str(config_group)

        branch.add(rich.syntax.Syntax(branch_content, "yaml"))

    rich.print(tree)

    # with open("config_tree.log", "w") as file:
    #     rich.print(tree, file=file)


@rank_zero_only
def log_hyperparameters(
    config: DictConfig,
    model: pl.LightningModule,
    datamodule: pl.LightningDataModule,
    trainer: pl.Trainer,
    callbacks: List[pl.Callback],
    logger: List[pl.loggers.base.LightningLoggerBase],
) -> None:
    """Controls which config parts are saved by Lightning loggers.

    Additionaly saves:
    - number of model parameters
    """

    if not trainer.logger:
        return

    hparams = {}

    # choose which parts of hydra config will be saved to loggers
    hparams["model"] = config["model"]

    # save number of model parameters
    hparams["model/params/total"] = sum(p.numel() for p in model.parameters())
    hparams["model/params/trainable"] = sum(
        p.numel() for p in model.parameters() if p.requires_grad
    )
    hparams["model/params/non_trainable"] = sum(
        p.numel() for p in model.parameters() if not p.requires_grad
    )

    hparams["datamodule"] = config["datamodule"]
    hparams["trainer"] = config["trainer"]

    if "seed" in config:
        hparams["seed"] = config["seed"]
    if "callbacks" in config:
        hparams["callbacks"] = config["callbacks"]

    # send hparams to all loggers
    trainer.logger.log_hyperparams(hparams)


def finish(
    config: DictConfig,
    model: pl.LightningModule,
    datamodule: pl.LightningDataModule,
    trainer: pl.Trainer,
    callbacks: List[pl.Callback],
    logger: List[pl.loggers.LightningLoggerBase],
) -> None:
    """Makes sure everything closed properly."""

    # without this sweeps with wandb logger might crash!
    for lg in logger:
        if isinstance(lg, pl.loggers.wandb.WandbLogger):
            import wandb

            wandb.finish()



import torch


def encode_passages(batch_text_passages, tokenizer, max_length):
    passage_ids, passage_masks = [], []
    max_len = max([len(text) for text in batch_text_passages])
    for text in batch_text_passages:
        text.extend([""]*(max_len-len(text)))

    for k, text_passages in enumerate(batch_text_passages):
        p = tokenizer.batch_encode_plus(
            text_passages,
            max_length=max_length,
            pad_to_max_length=True,
            return_tensors='pt',
            truncation=True
        )
        passage_ids.append(p['input_ids'][None])
        passage_masks.append(p['attention_mask'][None])

    passage_ids = torch.cat(passage_ids, dim=0)
    passage_masks = torch.cat(passage_masks, dim=0)
    return passage_ids, passage_masks.bool()


class DataCollatorForFid(object):
    def __init__(self, text_maxlength, tokenizer, max_target_length=128):
        self.tokenizer = tokenizer
        self.text_maxlength = text_maxlength
        self.max_target_length = max_target_length

    def __call__(self, batch):
        # assert (batch[0]['target'] != None)
        # print(batch)
        if "labels" in batch[0].keys():
            target = [ex['labels'] for ex in batch]
            target = self.tokenizer.batch_encode_plus(
                target,
                max_length=self.max_target_length if self.max_target_length > 0 else None,
                pad_to_max_length=True,
                return_tensors='pt',
                truncation=True if self.max_target_length > 0 else False,
            )
            target_ids = target["input_ids"]
            target_mask = target["attention_mask"].bool()
            target_ids = target_ids.masked_fill(~target_mask, -100)
        else:
            target_ids = None
            target_mask = None
        def append_question(example):
            return ["summarize: " + t for t in example['paragraphs']]

        text_passages = [append_question(example) for example in batch]
        passage_ids, passage_masks = encode_passages(text_passages,
                                                     self.tokenizer,
                                                     self.text_maxlength)

        return (target_ids, target_mask, passage_ids, passage_masks)



import argparse
import json
import os
from typing import List

import nltk

from SciAssist import BASE_OUTPUT_DIR, BASE_TEMP_DIR
from SciAssist.bin.doc2json.doc2json.grobid2json.process_pdf import process_pdf_file


def get_reference(json_file: str, output_dir: str = BASE_OUTPUT_DIR):

    os.makedirs(output_dir, exist_ok=True)
    assert json_file[-4:] == "json"
    output_path = os.path.join(output_dir,os.path.basename(json_file)[:-5]+"_ref.txt")

    strings_file = open(output_path,"w")

    with open(json_file,'r') as f:
        data = json.load(f)
        for _, context in data["pdf_parse"]["bib_entries"].items():
            raw_text = context["raw_text"]
            if raw_text != "":
                strings_file.write(raw_text)
                strings_file.write("\n")

    strings_file.close()
    return output_path


def get_bodytext(json_file: str, output_dir: str = BASE_OUTPUT_DIR, suffix="_body"):

    os.makedirs(output_dir, exist_ok=True)
    assert json_file[-4:] == "json"
    output_path = os.path.join(output_dir,os.path.basename(json_file)[:-5]+ suffix + ".txt")

    strings_file = open(output_path,"w")

    with open(json_file,'r') as f:
        data = json.load(f)
        for context in data["pdf_parse"]["body_text"]:
            sent = context["text"]
            if sent != "":
                sent = nltk.word_tokenize(sent)
                strings_file.write(" ".join(sent))
                strings_file.write(" ")

    strings_file.close()
    return output_path

def get_text_by_section(json_file: str, section: List, output_dir: str = BASE_OUTPUT_DIR, suffix=None):
    os.makedirs(output_dir, exist_ok=True)
    assert json_file[-4:] == "json"
    if suffix is None:
        suffix = "_" + "_".join(section)
    output_path = os.path.join(output_dir, os.path.basename(json_file)[:-5] + suffix + ".txt")
    strings_file = open(output_path,"w")

    section = [s.lower() for s in section]
    with open(json_file,'r') as f:
        data = json.load(f)
        for context in data["pdf_parse"]["body_text"]:
            sent = context["text"]
            for j in section:
                if sent != "" and j in context["section"].lower():
                    sent = nltk.word_tokenize(sent)
                    strings_file.write(" ".join(sent))
                    strings_file.write(" ")
                else:
                    continue

    strings_file.close()
    return output_path

def get_IC(json_file: str, output_dir: str = BASE_OUTPUT_DIR, suffix=None):
    os.makedirs(output_dir, exist_ok=True)
    section=["Introduction","Conclusion"]
    assert json_file[-4:] == "json"
    if suffix is None:
        suffix = "_" + "_".join(section)
    output_path = os.path.join(output_dir, os.path.basename(json_file)[:-5] + suffix + ".txt")
    strings_file = open(output_path,"w")
    res=[]
    section = [s.lower() for s in section]
    flag=0
    with open(json_file,'r') as f:
        data = json.load(f)
        for context in data["pdf_parse"]["body_text"]:
            sent = context["text"]
            for j in section:
                if sent != "" and j in context["section"].lower():
                    if j=="introduction":
                        flag=1
                    sent = nltk.word_tokenize(sent)
                    res.extend(sent)
                else:
                    continue
        if flag==0:
            res=[]
            for context in data["pdf_parse"]["body_text"]:
                sent = context["text"]
                if sent != "":
                    sent = nltk.word_tokenize(sent)
                    res.extend(sent)
    strings_file.write(" ".join(res[:800]))
    strings_file.write(" ")
    strings_file.close()
    return output_path


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Extracting strings from JSON")
    parser.add_argument("--input_file", required=True, help="path to the input json file")
    parser.add_argument("--temp_dir", default=BASE_TEMP_DIR, help="path to the temp dir for putting json files")
    parser.add_argument("--output_dir", default=BASE_OUTPUT_DIR, help="path to the output dir for putting text files")
    parser.add_argument("--save_temp", action="store_true", default=True, help="to save temporary json files")

    parser.add_argument("--reference", action="store_true", default=False, help="to extract reference strings")
    parser.add_argument("--bodytext", action="store_true", default=False, help="to extract bodytext")

    args = parser.parse_args()
    pdf_file = args.input_file
    output_dir = args.output_dir
    temp_dir = args.temp_dir


    json_file = process_pdf_file(input_file=pdf_file, temp_dir=temp_dir, output_dir=temp_dir)

    if args.reference:
        ref = get_reference(json_file=json_file, output_dir=output_dir)

    if args.bodytext:
        ref = get_bodytext(json_file=json_file, output_dir=output_dir)


# main developer: Yixi Ding <dingyixi@hotmail.com>

from typing import Dict

import torch

from SciAssist import BASE_CACHE_DIR
from SciAssist.models.components.bert_token_classifier import BertForTokenClassifier
from SciAssist.models.components.flant5_summarization import FlanT5ForSummarization
from SciAssist.models.components.bert_dataset_extraction import BertForDatasetExtraction
from SciAssist.utils.data_utils import (
    DataUtilsForTokenClassification,
    DataUtilsForSeq2Seq, DataUtilsForFlanT5, DataUtilsForT5,
    DataUtilsForSeq2Seq,
    DataUtilsForDatasetExtraction
)

# Provided models for each task
TASKS = {
    "reference-string-parsing": {
        "scibert-on-cora":  {
            "model": BertForTokenClassifier,
            "model_dict_url": "https://huggingface.co/spaces/wing-nus/SciAssist/resolve/main/scibert-uncased.pt",
            "data_utils": DataUtilsForTokenClassification
        },
        "default": {
            "model": BertForTokenClassifier,
            "model_dict_url": "https://huggingface.co/spaces/wing-nus/SciAssist/resolve/main/scibert-uncased.pt",
            "data_utils": DataUtilsForTokenClassification,
        },
    },

    "single-doc-summarization": {
        # "bart-cnn-on-mup": {
        #     "model": FlanT5ForSummarization,
        #     "model_dict_url": "https://huggingface.co/spaces/dyxohjl666/Controlled-summarization/resolve/main/flant5-mup-scisumm.pt",
        #     "data_utils": DataUtilsForSeq2Seq,
        # },
        "default": {
            "model": FlanT5ForSummarization,
            "model_dict_url": "https://huggingface.co/spaces/dyxohjl666/Controlled-summarization/resolve/main/flant5-base.pt",
            "data_utils": DataUtilsForSeq2Seq,
        },
        "t5": {
            "model": FlanT5ForSummarization,
            "model_dict_url": None,
            "data_utils": DataUtilsForT5,
        },
        "flan-t5": {
            "model": FlanT5ForSummarization,
            "model_dict_url": "https://huggingface.co/spaces/dyxohjl666/Controlled-summarization/resolve/main/flant5-base.pt",
            "data_utils": DataUtilsForFlanT5,
        }
    },

    "controlled-summarization": {
        "default": {
            "model": FlanT5ForSummarization,
            "model_dict_url": "https://huggingface.co/spaces/dyxohjl666/Controlled-summarization/resolve/main/flant5-mup-scisumm.pt",
            # "model_dict_url": None,
            "data_utils": DataUtilsForFlanT5,
        }
    },
    "dataset-extraction": {
        "default": {
            "model": BertForDatasetExtraction,
            "model_dict_url": "https://huggingface.co/spaces/wing-nus/SciAssist/resolve/main/dataset-extraction.pt",
            "data_utils": DataUtilsForDatasetExtraction,
        },
    },

    # "controlled-summarization": {
    #     "default": {
    #         "model": FrostForSummarization,
    #         "model_dict_url": None,
    #         "data_utils": DataUtilsForFrost,
    #     }
    # }

}

def load_model(config: Dict, cache_dir=BASE_CACHE_DIR, device="gpu"):
    '''

    Args:
        config (Dict):
            A dictionary including the model class and the url to download the weights dict.
            For example:
                {
                    "model": BertForTokenClassifier,
                    "model_dict_url": "https://huggingface.co/spaces/wing-nus/SciAssist/resolve/main/scibert-uncased.pt",
                }

    Returns: A loaded model.

    '''

    print("Loading the model...")
    model_class = config["model"]
    model = model_class(cache_dir=cache_dir)
    map_location = None if torch.cuda.is_available() and device in ["gpu","cuda"] else torch.device("cpu")
    if config["model_dict_url"]!=None:
        state_dict = torch.hub.load_state_dict_from_url(config["model_dict_url"], model_dir=cache_dir, map_location=map_location)
        model.load_state_dict(state_dict)
    else:
        # You can also choose to load your local trained model:
        # model.load_state_dict(torch.load("/home/linxiao/SciAssist-scibert-0223/src/models/scibert_ner/2023-03-25_02-27-17/scibert_dataset_extraction.pt"))
        pass

    model.eval()

    return model


# Copyright (c) Facebook, Inc. and its affiliates.
# All rights reserved.
#
# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import types
import torch
import transformers
import torch.nn.functional as F
from torch import nn


class FiDT5(transformers.T5ForConditionalGeneration):
    def __init__(self, model_checkpoint="google/flan-t5-large",
                 cache_dir: str = ".cache",
                 save_name: str = "long-flant5-mup.pt",
                 model_dir: str = "pretrained"
    ):

        t5 = transformers.AutoModelForSeq2SeqLM.from_pretrained(model_checkpoint, cache_dir=cache_dir)
        super().__init__(t5.config)
        self.wrap_encoder()

        self.save_name = save_name
        self.model_dir = model_dir

        print(f"The model checkpoint are cached in '{cache_dir}'.")

    def forward_(self, **kwargs):
        if 'input_ids' in kwargs:
            kwargs['input_ids'] = kwargs['input_ids'].view(kwargs['input_ids'].size(0), -1)
        if 'attention_mask' in kwargs:
            kwargs['attention_mask'] = kwargs['attention_mask'].view(kwargs['attention_mask'].size(0), -1)

        return super(FiDT5, self).forward(
            **kwargs
        )

    # We need to resize as B x (N * L) instead of (B * N) x L here
    # because the T5 forward method uses the input tensors to infer
    # dimensions used in the decoder.
    # EncoderWrapper resizes the inputs as (B * N) x L.
    def forward(self, input_ids=None, attention_mask=None, **kwargs):
        if input_ids != None:
            # inputs might have already be resized in the generate method
            if input_ids.dim() == 3:
                self.encoder.n_passages = input_ids.size(1)
            input_ids = input_ids.view(input_ids.size(0), -1)
        if attention_mask != None:
            attention_mask = attention_mask.view(attention_mask.size(0), -1)
        return super().forward(
            input_ids=input_ids,
            attention_mask=attention_mask,
            **kwargs
        )

    # We need to resize the inputs here, as the generate method expect 2D tensors
    def generate(self, input_ids, attention_mask, max_length=128, num_beams=1, num_return_sequences=1):
        diversity_penalty = 0.0
        if num_return_sequences>1:
            diversity_penalty = 1.0

        self.encoder.n_passages = input_ids.size(1)
        return super().generate(
            input_ids=input_ids.view(input_ids.size(0), -1),
            attention_mask=attention_mask.view(attention_mask.size(0), -1),
            max_length=max_length,
            num_beams=num_beams,
            num_return_sequences=num_return_sequences,
            num_beam_groups=num_return_sequences,
            diversity_penalty=diversity_penalty
        )

    def wrap_encoder(self, use_checkpoint=False):
        """
        Wrap T5 encoder to obtain a Fusion-in-Decoder model.
        """
        self.encoder = EncoderWrapper(self.encoder, use_checkpoint=use_checkpoint)

    def unwrap_encoder(self):
        """
        Unwrap Fusion-in-Decoder encoder, useful to load T5 weights.
        """
        self.encoder = self.encoder.encoder
        block = []
        for mod in self.encoder.block:
            block.append(mod.module)
        block = nn.ModuleList(block)
        self.encoder.block = block

    def load_t5(self, state_dict):
        self.unwrap_encoder()
        self.load_state_dict(state_dict)
        self.wrap_encoder()

    def set_checkpoint(self, use_checkpoint):
        """
        Enable or disable checkpointing in the encoder.
        See https://pytorch.org/docs/stable/checkpoint.html
        """
        for mod in self.encoder.encoder.block:
            mod.use_checkpoint = use_checkpoint

    def reset_score_storage(self):
        """
        Reset score storage, only used when cross-attention scores are saved
        to train a retriever.
        """
        for mod in self.decoder.block:
            mod.layer[1].EncDecAttention.score_storage = None

    def get_crossattention_scores(self, context_mask):
        """
        Cross-attention scores are aggregated to obtain a single scalar per
        passage. This scalar can be seen as a similarity score between the
        question and the input passage. It is obtained by averaging the
        cross-attention scores obtained on the first decoded token over heads,
        layers, and tokens of the input passage.

        More details in Distilling Knowledge from Reader to Retriever:
        https://arxiv.org/abs/2012.04584.
        """
        scores = []
        n_passages = context_mask.size(1)
        for mod in self.decoder.block:
            scores.append(mod.layer[1].EncDecAttention.score_storage)
        scores = torch.cat(scores, dim=2)
        bsz, n_heads, n_layers, _ = scores.size()
        # batch_size, n_head, n_layers, n_passages, text_maxlength
        scores = scores.view(bsz, n_heads, n_layers, n_passages, -1)
        scores = scores.masked_fill(~context_mask[:, None, None], 0.)
        scores = scores.sum(dim=[1, 2, 4])
        ntokens = context_mask.sum(dim=[2]) * n_layers * n_heads
        scores = scores/ntokens
        return scores

    def overwrite_forward_crossattention(self):
        """
        Replace cross-attention forward function, only used to save
        cross-attention scores.
        """
        for mod in self.decoder.block:
            attn = mod.layer[1].EncDecAttention
            attn.forward = types.MethodType(cross_attention_forward, attn)

class EncoderWrapper(torch.nn.Module):
    """
    Encoder Wrapper for T5 Wrapper to obtain a Fusion-in-Decoder model.
    """
    def __init__(self, encoder, use_checkpoint=False):
        super().__init__()

        self.encoder = encoder
        self.main_input_name = "input_ids"
        apply_checkpoint_wrapper(self.encoder, use_checkpoint)

    def forward(self, input_ids=None, attention_mask=None, **kwargs,):
        # total_length = n_passages * passage_length
        bsz, total_length = input_ids.shape
        passage_length = total_length // self.n_passages
        input_ids = input_ids.view(bsz*self.n_passages, passage_length)
        attention_mask = attention_mask.view(bsz*self.n_passages, passage_length)
        outputs = self.encoder(input_ids, attention_mask, **kwargs)
        outputs = (outputs[0].view(bsz, self.n_passages*passage_length, -1), ) + outputs[1:]
        return outputs

class CheckpointWrapper(torch.nn.Module):
    """
    Wrapper replacing None outputs by empty tensors, which allows the use of
    checkpointing.
    """
    def __init__(self, module, use_checkpoint=False):
        super().__init__()
        self.module = module
        self.use_checkpoint = use_checkpoint

    def forward(self, hidden_states, attention_mask, position_bias, **kwargs):
        if self.use_checkpoint and self.training:
            kwargs = {k: v for k, v in kwargs.items() if v is not None}
            def custom_forward(*inputs):
                output = self.module(*inputs, **kwargs)
                empty = torch.tensor(
                    [],
                    dtype=torch.float,
                    device=output[0].device,
                    requires_grad=True)
                output = tuple(x if x is not None else empty for x in output)
                return output

            output = torch.utils.checkpoint.checkpoint(
                custom_forward,
                hidden_states,
                attention_mask,
                position_bias
            )
            output = tuple(x if x.size() != 0 else None for x in output)
        else:
            output = self.module(hidden_states, attention_mask, position_bias, **kwargs)
        return output

def apply_checkpoint_wrapper(t5stack, use_checkpoint):
    """
    Wrap each block of the encoder to enable checkpointing.
    """
    block = []
    for mod in t5stack.block:
        wrapped_mod = CheckpointWrapper(mod, use_checkpoint)
        block.append(wrapped_mod)
    block = nn.ModuleList(block)
    t5stack.block = block

def cross_attention_forward(
        self,
        input,
        mask=None,
        kv=None,
        position_bias=None,
        past_key_value_state=None,
        head_mask=None,
        query_length=None,
        use_cache=False,
        output_attentions=False,
    ):
    """
    This only works for computing cross attention over the input
    """
    assert(kv != None)
    assert(head_mask == None)
    assert(position_bias != None or self.has_relative_attention_bias)

    bsz, qlen, dim = input.size()
    n_heads, d_heads = self.n_heads, self.d_kv
    klen = kv.size(1)

    q = self.q(input).view(bsz, -1, n_heads, d_heads).transpose(1, 2)
    if past_key_value_state == None:
        k = self.k(kv).view(bsz, -1, n_heads, d_heads).transpose(1, 2)
        v = self.v(kv).view(bsz, -1, n_heads, d_heads).transpose(1, 2)
    else:
        k, v = past_key_value_state

    scores = torch.einsum("bnqd,bnkd->bnqk", q, k)

    if mask is not None:
       scores += mask

    if position_bias is None:
        position_bias = self.compute_bias(qlen, klen)
    scores += position_bias

    if self.score_storage is None:
        self.score_storage = scores

    attn = F.softmax(scores.float(), dim=-1).type_as(scores)
    attn = F.dropout(attn, p=self.dropout, training=self.training)

    output = torch.matmul(attn, v)
    output = output.transpose(1, 2).contiguous().view(bsz, -1, self.inner_dim)
    output = self.o(output)

    if use_cache:
        output = (output,) + ((k, v),)
    else:
        output = (output,) + (None,)

    if output_attentions:
        output = output + (attn,)

    if self.has_relative_attention_bias:
        output = output + (position_bias,)

    return output





# main developer: Yixi Ding <dingyixi@hotmail.com>

import json
import os
from typing import List, Tuple, Optional, Union, Dict

from datasets import Dataset
from transformers import PreTrainedTokenizer

from SciAssist import BASE_OUTPUT_DIR, BASE_TEMP_DIR
from SciAssist.datamodules.components.cora_label import LABEL_NAMES
from SciAssist.datamodules.components.cora_label import label2id
from SciAssist.pipelines.pipeline import Pipeline
from SciAssist.utils.pdf2text import process_pdf_file, get_reference
from SciAssist.utils.windows_pdf2text import windows_get_reference


class ReferenceStringParsing(Pipeline):
    """
    The pipeline for reference string parsing.

    Args:
        model_name (`str`, *optional*):
            A string, the *model name* of a pretrained model provided for this task.
        device (`str`, *optional*):
            A string, `cpu` or `gpu`.
        cache_dir (`str` or `os.PathLike`, *optional*):
            Path to a directory in which a downloaded pretrained model should be
            cached if the standard cache should not be used.
        output_dir (`str` or `os.PathLike`, *optional*):
            Path to a directory in which the predicted results files should be stored.
        temp_dir (`str` or `os.PathLike`, *optional*):
            Path to a directory which holds temporary files such as `.tei.xml`.
        tokenizer (PreTrainedTokenizer, *optional*):
            A specific tokenizer.
        checkpoint (`str` or `os.PathLike`, *optional*):
            A checkpoint for the tokenizer. You can also specify the `checkpoint` while
            using the default tokenizer.
            Can be either:

                - A string, the *model id* of a predefined tokenizer hosted inside a model repo on huggingface.co.
                  Valid model ids can be located at the root-level, like `bert-base-uncased`, or namespaced under a
                  user or organization name, like `allenai/scibert_scivocab_uncased`.
                - A path to a *directory* containing vocabulary files required by the tokenizer, for instance saved
                  using the [`~PreTrainedTokenizer.save_pretrained`] method, e.g., `./my_model_directory/`.
                - A path or url to a single saved vocabulary file if and only if the tokenizer only requires a
                  single vocabulary file (like Bert or XLNet), e.g.: `./my_model_directory/vocab.txt`. (Not
                  applicable to all derived classes)

        model_max_length (`int`, *optional*): The max sequence length the model accepts.
    """

    def __init__(
            self, model_name: Optional[str] = "default", device: Optional[str] = "gpu",
            cache_dir = None,
            output_dir = None,
            temp_dir = None,
            tokenizer: PreTrainedTokenizer = None,
            checkpoint="allenai/scibert_scivocab_uncased",
            model_max_length=512,
            os_name=None,
    ):

        super().__init__(task_name="reference-string-parsing", model_name=model_name, device=device,
                         cache_dir=cache_dir, output_dir=output_dir, temp_dir=temp_dir)

        self.data_utils = self.data_utils(
            tokenizer=tokenizer,
            checkpoint=checkpoint,
            model_max_length=model_max_length
        )
        self.os_name = os_name if os_name != None else os.name

    def predict(
            self, input, type: str = "pdf", dehyphen=False,
            output_dir=None,
            temp_dir=None,
            save_results=True,
    ):

        """

        Args:
            input (`str` or `List[str]` or `os.PathLike`):
                Can be either:

                    - A string, the reference string to be parsed.
                    - A list of strings to be parsed.
                    - A path to a *.txt* file to be parsed. Each line of the source file contains a
                      reference string.
                    - A path to a *.pdf* file to be parsed, a raw scientific document without processing.
                      The pipeline will automatically extract the reference strings from the pdf.

            type (`str`, default to `pdf`):
                The type of input, can be either:

                    - `str` or `string`.
                    - `text`or `txt` for a .txt file.
                    - `pdf` for a pdf file. This is the default value.

            dehyphen (`bool`, default to `False`):
                Whether to remove hyphens in raw text.
            output_dir (`str` or `os.PathLike`, *optional*):
                Path to a directory in which the predicted results files should be stored.
                If not provided, it will use the `output_dir` set for the pipeline.
            temp_dir (`str` or `os.PathLike`, *optional*):
                Path to a directory which holds temporary files such as `.tei.xml`.
                If not provided, it will use the `temp_dir` set for the pipeline.
            save_results (`bool`, default to `True`):
                Whether to save the results in a *.json* file.
                **Note**: This is invalid when `type` is set to `str` or `string`.

        Returns:
            `List[Dict]`: [{"tagged_text": tagged_text, "tokens": tokens_list ,"tags": tags_list } , ... ]


        Examples:

            >>> from SciAssist import ReferenceStringParsing
            >>> pipeline = ReferenceStringParsing()
            >>> pipeline.predict(
            ...     "Waleed Ammar, Matthew E. Peters, Chandra Bhagavat- ula, and Russell Power. 2017. The ai2 system at semeval-2017 task 10 (scienceie): semi-supervised end-to-end entity and relation extraction. In ACL workshop (SemEval).",
            ...     type="str"
            ... )
            [{'tagged_text': '<author>Waleed</author> <author>Ammar,</author> <author>Matthew</author> <author>E.</author> <author>Peters,</author> <author>Chandra</author> <author>Bhagavat-</author> <author>ula,</author> <author>and</author> <author>Russell</author> <author>Power.</author> <date>2017.</date> <title>The</title> <title>ai2</title> <title>system</title> <title>at</title> <title>semeval-2017</title> <title>task</title> <title>10</title> <title>(scienceie):</title> <title>semi-supervised</title> <title>end-to-end</title> <title>entity</title> <title>and</title> <title>relation</title> <title>extraction.</title> <booktitle>In</booktitle> <booktitle>ACL</booktitle> <booktitle>workshop</booktitle> <booktitle>(SemEval).</booktitle>',
            'tokens': ['Waleed', 'Ammar,', 'Matthew', 'E.', 'Peters,', 'Chandra', 'Bhagavat-', 'ula,', 'and', 'Russell', 'Power.', '2017.', 'The', 'ai2', 'system', 'at', 'semeval-2017', 'task', '10', '(scienceie):', 'semi-supervised', 'end-to-end', 'entity', 'and', 'relation', 'extraction.', 'In', 'ACL', 'workshop', '(SemEval).'],
            'tags': ['author', 'author', 'author', 'author', 'author', 'author', 'author', 'author', 'author', 'author', 'author', 'date', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'title', 'booktitle', 'booktitle', 'booktitle', 'booktitle']}]

        """
        if output_dir is None:
            output_dir = self.output_dir
        if temp_dir is None:
            temp_dir = self.temp_dir

        if type in ["str", "string"]:
            results = self._predict_for_string(example=input, dehyphen=dehyphen)
        elif type in ["txt", "text"]:
            results = self._predict_for_text(filename=input, dehyphen=dehyphen)
        elif type == "pdf":
            results = self._predict_for_pdf(filename=input, output_dir=output_dir, temp_dir=temp_dir, dehyphen=dehyphen)
        # Save predicted results as a text file
        if save_results and type not in ["str", "string"]:
            os.makedirs(output_dir, exist_ok=True)
            output_file = os.path.basename(input)
            with open(os.path.join(output_dir, f"{output_file[:-4]}_rsp.json"), "w") as output:
                for res in results:
                    output.write(json.dumps(res) + "\n")

        return results

    def _dehyphen_for_str(self, text: str):
        text = text.replace("- ", "")
        text = text.replace("-", " ")
        return text

    def _to_device(self, batch):
        if self.model_name in ["default", "scibert-on-cora"]:
            return {
                "input_ids": batch["input_ids"].to(self.device),
                "token_type_ids": batch["token_type_ids"].to(self.device),
                "attention_mask": batch["attention_mask"].to(self.device),
                "token_mapping": batch["token_mapping"].to(self.device)
            }

    def _predict(self, examples: List[List[str]]) -> List[Dict]:
        """
        Parse a list of tokens obtained from reference strings.

        Args:
            examples (`List[List[str]]`):
                The inputs for inference, where each item is a list of tokens.

        Returns:
            `List[Dict]`:
                Tagged strings, origin tokens and labels predicted by the model.

        """

        # Prepare the dataset
        dict_data = {"tokens": examples}
        dataset = Dataset.from_dict(dict_data)

        dataloader = self.data_utils.get_dataloader(dataset, label2id=label2id)

        results = []
        true_preds = []
        for batch in dataloader:
            # Predict the labels

            batch = self._to_device(batch)
            outputs = self.model(**batch)
            preds = outputs.logits.argmax(dim=-1)
            # Convert ids to labels and
            # merge the labels according to origin tokens.
            preds = [[LABEL_NAMES[i] for i in pred if i < len(LABEL_NAMES)] for pred in preds]
            true_preds.extend(preds)

        tokens = examples

        # Generate the tagged strings.
        for i in range(len(tokens)):
            tagged_words = []
            for token, label in zip(tokens[i], true_preds[i]):
                tagged_word = f"<{label}>{token}</{label}>"
                tagged_words.append(tagged_word)
            result = " ".join(tagged_words)
            results.append(
                {
                    "tagged_text": result,
                    "tokens": tokens[i],
                    "tags": true_preds[i],
                }
            )
        return results

    def _predict_for_string(self, example: Union[str, List[str]], dehyphen: Optional[bool] = False) -> List[Dict]:
        """
        Parse a reference string.

        Args:
            example (`Union[str, List[str]]`): The string to parse.
            dehyphen (`Optional[bool]`): Whether to remove '-', default to `False`.
        Returns:
           `List[Dict]`:
                Tagged string, origin tokens and labels predicted by the model.

        """

        if isinstance(example, list):
            examples = example
        else:
            examples = [example]

        # remove '-' in text
        if dehyphen == True:
            examples = [self._dehyphen_for_str(example) for example in examples]

        splitted_examples = [example.split() for example in examples]
        results = self._predict(splitted_examples)

        return results

    def _predict_for_text(
            self,
            filename: str,
            dehyphen: Optional[bool] = False,
    ) -> List[Dict]:
        """

        Parse reference strings from a text and save the result as a text file.

        Args:
            filename (`str`): The path to the text file to predict.
            dehyphen (`Optional[bool]`): Whether to remove '-', default to `False`.

        Returns:
            `List[Dict]`:
                Tagged strings, origin tokens and labels predicted by the model.

        """

        with open(filename, "r") as f:
            examples = f.readlines()

        # remove '-' in text
        if dehyphen == True:
            examples = [self._dehyphen_for_str(example) for example in examples]

        splitted_examples = [example.split() for example in examples]
        results = self._predict(splitted_examples)

        return results

    def _predict_for_pdf(
            self,
            filename: str,
            output_dir: Optional[str] = BASE_OUTPUT_DIR,
            temp_dir: Optional[str] = BASE_TEMP_DIR,
            dehyphen: Optional[bool] = False,
    ) -> Tuple[List[str], List[List[str]], List[List[str]]]:
        """
        Parse reference strings from a PDF and save the result as a text file.

        Args:
            filename (`str`): The path to the pdf file to parse.
            output_dir (`Optional[str]`): The directory to save the result file, default to `result/`.
            temp_dir (`Optional[str]`): The diretorcy to save intermediate file, default to `temp/`.
            dehyphen (`Optional[bool]`): Whether to remove '-', default to `False`.

        Returns:
           `List[Dict]`:
                Tagged strings, origin tokens and labels predicted by the model.
        """
        if self.os_name == "posix":
            # Convert PDF to JSON with doc2json.
            json_file = process_pdf_file(input_file=filename, temp_dir=temp_dir, output_dir=temp_dir)
            # Extract reference strings from JSON and save them in TEXT format.
            text_file = get_reference(json_file=json_file, output_dir=output_dir)
        elif self.os_name == "nt":
            text_file = windows_get_reference(path=filename, output_dir=output_dir)


        return self._predict_for_text(text_file, dehyphen=dehyphen)



# main developer: Yixi Ding <dingyixi@hotmail.com>
import csv
import math
import os
import random
from pathlib import Path
from typing import Optional

import datasets
from datasets import Dataset, DatasetDict
from pytorch_lightning import LightningDataModule
from torch.utils.data import DataLoader, random_split

from SciAssist.utils.data_reader import csv_reader
from SciAssist.utils.data_utils import DataUtilsForSeq2Seq


class MupSciSummDataModule(LightningDataModule):
    def __init__(
        self,
        data_repo: str,
        train_batch_size: int = 8,
        num_workers: int = 0,
        pin_memory: bool = False,
        data_cache_dir: str = ".cache",
        seed: int = 777,
        data_utils = DataUtilsForSeq2Seq
    ):
        super().__init__()
        self.save_hyperparameters(logger=False)
        self.data_cache_dir = Path(self.hparams.data_cache_dir)
        self.data_utils = self.hparams.data_utils
        self.data_collator = self.data_utils.collator()

        self.data_train: Optional[Dataset] = None
        self.data_val: Optional[Dataset] = None
        self.data_test: Optional[Dataset] = None

    # def prepare_data(self) -> DatasetDict:
    #     mup_datasets  = datasets.load_dataset(
    #         self.hparams.data_repo,
    #         cache_dir=self.data_cache_dir
    #     )
    #     # Prepare keywords
    #     id2kw = {}
    #     with open("/home/dingyx/project/SciAssist/data/Task2/From-ScisummNet-2019/scisumm_flant5_target_Entities_sci2.csv", 'r', newline='', encoding='ISO-8859-1') as f:
    #         rows = csv.reader(f)
    #         # Get Column names
    #         keys = next(rows)
    #         # Add values by column
    #         for row in rows:
    #             id2kw[row[0]] = row[1]
    #
    #
    #     file_list = []
    #     root_dir = "/home/dingyx/project/SciAssist/data/Task2/From-ScisummNet-2019/"
    #     for dirpath, dirnames, files in os.walk(root_dir):
    #         file_list = dirnames
    #         break
    #
    #     texts = []
    #     summaries = []
    #     keywords = []
    #     for file in file_list:
    #         with open(os.path.join(root_dir, file, "summary", file + ".scisummnet_human.txt"), "r") as f:
    #             summary = f.readlines()
    #             summary = " ".join(summary[1:])
    #             summaries.append(summary)
    #         with open(os.path.join(root_dir, file, file + ".txt"), "r") as f:
    #             text = f.readlines()
    #             text = " ".join(text)
    #             texts.append(text)
    #         keywords.append(id2kw[file].split(","))
    #     scisumm_datasets = {
    #         "text": texts,
    #         "summary": summaries,
    #         "keywords": keywords,
    #         "length": [None for i in texts]
    #     }
    #
    #     lengths = [len(s.split(" ")) for s in mup_datasets["train"]["summary"]]
    #     lengths = [50*math.ceil(s/50) for s in lengths]
    #
    #     raw_datasets = DatasetDict()
    #     raw_datasets["train"] = {
    #         "text": scisumm_datasets["text"] + mup_datasets["train"]["text"],
    #         "summary": scisumm_datasets["summary"] + mup_datasets["train"]["summary"],
    #         "keywords": keywords + [None for i in mup_datasets["train"]["text"]],
    #         "length": scisumm_datasets["length"] + lengths
    #     }
    #
    #
    #
    #     raw_datasets["train"] = Dataset.from_dict(raw_datasets["train"])
    #     # raw_datasets["test"] = Dataset.from_dict(mup_datasets)
    #     # raw_datasets["validation"] = Dataset.from_dict(raw_datasets["validation"])
    #     # raw_datasets = Dataset.from_dict(raw_datasets)
    #     return raw_datasets


    def prepare_data(self) -> DatasetDict:

        # Prepare keywords
        summ = []
        kw = []
        text = []
        with open("/home/yixi/project/sciassist/data/train2.csv", 'r', newline='', encoding='ISO-8859-1') as f:
            rows = csv.reader(f)
            # Get Column names
            keys = next(rows)
            # Add values by column
            for row in rows:
                if row[0].isnumeric():
                    kws = row[4].split(",")
                    kws = list(set(kws))
                    summ.append(row[2])
                    kw.append(kws)
                    text.append(row[3])
                # lengths.append(100)

        lengths = [len(s.split(" ")) for s in summ]
        lengths = [50*math.ceil(s/50) for s in lengths]

        mask1=[]
        mask2=[]
        mask1 = random.sample(range(0, int(len(text))), int(len(text) / 3))
        mask2 = random.sample(range(0, int(len(text))), int(len(text) / 5))

        # for i in range(int(len(text)/3)):
        #     mask2.append(random.randint(0, len(text)-1))

        for i in mask1:
            if len(kw[i])<=1:
                kw[i] = None
        for i in mask2:
            if lengths[i] <= 100:
                lengths[i] = None

        raw_datasets = DatasetDict()
        raw_datasets["train"] = {
            "text": text,
            "summary": summ,
            "keywords": kw,
            "length": lengths
        }



        raw_datasets["train"] = Dataset.from_dict(raw_datasets["train"])
        # raw_datasets["test"] = Dataset.from_dict(mup_datasets)
        # raw_datasets["validation"] = Dataset.from_dict(raw_datasets["validation"])
        # raw_datasets = Dataset.from_dict(raw_datasets)
        return raw_datasets



    def setup(self, stage: Optional[str] = None):
        if not self.data_train and not self.data_val and not self.data_test:
            processed_datasets = self.prepare_data()
            tokenized_datasets = processed_datasets.map(
                lambda x: self.data_utils.tokenize_and_align_labels(x, inputs_column="text", labels_column="summary"),
                batched=True,
                remove_columns=processed_datasets["train"].column_names,
                load_from_cache_file=True
            )
            # self.data_train = tokenized_datasets["train"]
            # self.data_val = tokenized_datasets["validation"]
            length = len(tokenized_datasets["train"])
            train_size = int(0.9 * length)
            validate_size = length - train_size
            self.data_train, self.data_val = random_split(tokenized_datasets["train"], [train_size, validate_size])
            # self.data_test = tokenized_datasets["test"].select(range(100))
            # If labels are not provided, delete the column "labels"
            # self.data_test = tokenized_datasets["test"]

    def train_dataloader(self):
        return DataLoader(
            dataset=self.data_train,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_collator,
            shuffle=True,
        )

    def val_dataloader(self):
        return DataLoader(
            dataset=self.data_val,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_collator,
            shuffle=False,
        )
    #
    # def test_dataloader(self):
    #     return DataLoader(
    #         dataset=self.data_test,
    #         batch_size=self.hparams.train_batch_size,
    #         num_workers=self.hparams.num_workers,
    #         pin_memory=self.hparams.pin_memory,
    #         collate_fn=self.data_collator,
    #         shuffle=False,
    #     )
    #



# main developer: Yixi Ding <dingyixi@hotmail.com>
import csv
import math
import os
from pathlib import Path
from typing import Optional

import datasets
from datasets import Dataset, DatasetDict
from pytorch_lightning import LightningDataModule
from torch.utils.data import DataLoader, random_split

from SciAssist.utils.data_utils import DataUtilsForSeq2Seq


class MupSciSummDataModule(LightningDataModule):
    def __init__(
            self,
            data_repo: str,
            train_batch_size: int = 8,
            num_workers: int = 0,
            pin_memory: bool = False,
            data_cache_dir: str = ".cache",
            seed: int = 777,
            data_utils=DataUtilsForSeq2Seq
    ):
        super().__init__()
        self.save_hyperparameters(logger=False)
        self.data_cache_dir = Path(self.hparams.data_cache_dir)
        self.data_utils = self.hparams.data_utils
        self.data_collator = self.data_utils.collator()

        self.data_train: Optional[Dataset] = None
        self.data_val: Optional[Dataset] = None
        self.data_test: Optional[Dataset] = None

    def prepare_data(self) -> DatasetDict:
        mup_datasets = datasets.load_dataset(
            self.hparams.data_repo,
            cache_dir=self.data_cache_dir
        )
        # Prepare keywords
        id2kw = {}
        filter_files = ['C04-1046', 'J93-2004', 'C00-1072', 'J94-4004', 'C04-1100', 'C88-1016', 'M95-1005', 'J05-1004',
                        'H94-1046', 'C86-1016', 'P90-1010', 'C96-1079', 'P89-1009', 'C92-3150', 'C00-2136', 'C92-1025',
                        'H94-1048', 'M95-1012', 'H92-1026', 'P85-1011', 'C96-2183', 'C94-1042', 'C92-2082', 'C02-2025',
                        'H05-2018', 'W08-2123', 'C96-1058', 'H91-1060', 'C88-2128', 'C02-1139', 'C00-1007', 'C92-3126',
                        'I05-3025', 'J06-1003', 'J08-2005', 'H91-1026', 'C94-1032', 'C94-1027', 'H93-1051', 'C88-2147',
                        'H92-1045', 'C90-3063', 'C90-3044', 'C94-2174', 'D07-1074', 'H93-1052', 'N03-2021', 'C88-2121',
                        'C92-2070', 'C00-2137', 'W97-0703', 'H94-1020', 'C90-3030', 'H01-1035', 'C96-1055', 'H93-1061',
                        'C96-1005', 'C00-1044', 'C92-1038', 'C00-2163', 'C90-3052', 'J94-2003', 'C04-1073']
        texts = []
        summaries = []
        keywords = []
        with open("/home/yixi/project/scisumm-corpus/data/Training-Set-2019/Task2/From-ScisummNet-2019/forecite.csv",
                  'r', newline='', encoding='ISO-8859-1') as f:
            rows = csv.reader(f)
            # Get Column names
            keys = next(rows)
            # Add values by column
            for row in rows:
                if row[1].strip() == "":
                    id2kw[row[0]] = None
                else:
                    k_list = row[1].split(",")
                    id2kw[row[0]] = [k.strip() for k in k_list]

        file_list = []
        root_dir = "/home/yixi/project/scisumm-corpus/data/Training-Set-2019/Task2/From-ScisummNet-2019"
        for dirpath, dirnames, files in os.walk(root_dir):
            file_list = dirnames
            break

        for file in file_list:
            if file in filter_files:
                continue
            with open(os.path.join(root_dir, file, "summary", file + ".scisummnet_human.txt"), "r") as f:
                summary = f.readlines()
                summary = " ".join(summary[1:])
                summaries.append(summary)
            with open(os.path.join(root_dir, file, file + ".txt"), "r") as f:
                text = f.readlines()
                text = " ".join(text)
                texts.append(text)
            keywords.append(id2kw[file])
        scisumm_datasets = {
            "text": texts,
            "summary": summaries,
            "keywords": keywords,
            "length": [None for i in texts]
        }

        lengths = [len(s.split(" ")) for s in mup_datasets["train"]["summary"]]
        lengths = [50 * math.ceil(s / 50) for s in lengths]

        raw_datasets = DatasetDict()
        raw_datasets["train"] = {
            "text": scisumm_datasets["text"] + mup_datasets["train"]["text"],
            "summary": scisumm_datasets["summary"] + mup_datasets["train"]["summary"],
            "keywords": keywords + [None for i in mup_datasets["train"]["text"]],
            "length": scisumm_datasets["length"] + lengths
        }

        raw_datasets["train"] = Dataset.from_dict(raw_datasets["train"])
        # raw_datasets["test"] = Dataset.from_dict(mup_datasets)
        # raw_datasets["validation"] = Dataset.from_dict(raw_datasets["validation"])
        # raw_datasets = Dataset.from_dict(raw_datasets)
        return raw_datasets

    def setup(self, stage: Optional[str] = None):
        if not self.data_train and not self.data_val and not self.data_test:
            processed_datasets = self.prepare_data()
            tokenized_datasets = processed_datasets.map(
                lambda x: self.data_utils.tokenize_and_align_labels(x, inputs_column="text", labels_column="summary"),
                batched=True,
                remove_columns=processed_datasets["train"].column_names,
                load_from_cache_file=True
            )
            # self.data_train = tokenized_datasets["train"]
            # self.data_val = tokenized_datasets["validation"]
            length = len(tokenized_datasets["train"])
            train_size = int(0.9 * length)
            validate_size = length - train_size
            self.data_train, self.data_val = random_split(tokenized_datasets["train"], [train_size, validate_size])
            # self.data_test = tokenized_datasets["test"].select(range(100))
            # If labels are not provided, delete the column "labels"
            # self.data_test = tokenized_datasets["test"]

    def train_dataloader(self):
        return DataLoader(
            dataset=self.data_train,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_collator,
            shuffle=True,
        )

    def val_dataloader(self):
        return DataLoader(
            dataset=self.data_val,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_collator,
            shuffle=False,
        )
    #
    # def test_dataloader(self):
    #     return DataLoader(
    #         dataset=self.data_test,
    #         batch_size=self.hparams.train_batch_size,
    #         num_workers=self.hparams.num_workers,
    #         pin_memory=self.hparams.pin_memory,
    #         collate_fn=self.data_collator,
    #         shuffle=False,
    #     )
    #



from typing import List, Tuple


def replace_refspans(
    spans_to_replace: List[Tuple[int, int, str, str]],
    full_string: str,
    pre_padding: str = "",
    post_padding: str = "",
    btwn_padding: str = ", "
) -> str:
    """
    For each span within the full string, replace that span with new text
    :param spans_to_replace: list of tuples of form (start_ind, end_ind, span_text, new_substring)
    :param full_string:
    :param pre_padding:
    :param post_padding:
    :param btwn_padding:
    :return:
    """
    # assert all spans are equal to full_text span
    assert all([full_string[start:end] == span for start, end, span, _ in spans_to_replace])

    # assert none of the spans start with the same start ind
    start_inds = [rep[0] for rep in spans_to_replace]
    assert len(set(start_inds)) == len(start_inds)

    # sort by start index
    spans_to_replace.sort(key=lambda x: x[0])

    # form strings for each span group
    for i, entry in enumerate(spans_to_replace):
        start, end, span, new_string = entry

        # skip empties
        if end <= 0:
            continue

        # compute shift amount
        shift_amount = len(new_string) - len(span) + len(pre_padding) + len(post_padding)

        # shift remaining appropriately
        for ind in range(i + 1, len(spans_to_replace)):
            next_start, next_end, next_span, next_string = spans_to_replace[ind]
            # skip empties
            if next_end <= 0:
                continue
            # if overlap between ref span and current ref span, remove from replacement
            if next_start < end:
                next_start = 0
                next_end = 0
                next_string = ""
            # if ref span abuts previous reference span
            elif next_start == end:
                next_start += shift_amount
                next_end += shift_amount
                next_string = btwn_padding + pre_padding + next_string + post_padding
            # if ref span starts after, shift starts and ends
            elif next_start > end:
                next_start += shift_amount
                next_end += shift_amount
                next_string = pre_padding + next_string + post_padding
            # save adjusted span
            spans_to_replace[ind] = (next_start, next_end, next_span, next_string)

    spans_to_replace = [entry for entry in spans_to_replace if entry[1] > 0]
    spans_to_replace.sort(key=lambda x: x[0])

    # apply shifts in series
    for start, end, span, new_string in spans_to_replace:
        assert full_string[start:end] == span
        full_string = full_string[:start] + new_string + full_string[end:]

    return full_string


def sub_spans_and_update_indices(
    spans_to_replace: List[Tuple[int, int, str, str]],
    full_string: str
) -> Tuple[str, List]:
    """
    Replace all spans and recompute indices
    :param spans_to_replace:
    :param full_string:
    :return:
    """
    # TODO: check no spans overlapping
    # TODO: check all spans well-formed

    # assert all spans are equal to full_text span
    assert all([full_string[start:end] == token for start, end, token, _ in spans_to_replace])

    # assert none of the spans start with the same start ind
    start_inds = [rep[0] for rep in spans_to_replace]
    assert len(set(start_inds)) == len(start_inds)

    # sort by start index
    spans_to_replace.sort(key=lambda x: x[0])

    # compute offsets for each span
    new_spans = [[start, end, token, surface, 0] for start, end, token, surface in spans_to_replace]
    for i, entry in enumerate(spans_to_replace):
        start, end, token, surface = entry
        new_end = start + len(surface)
        offset = new_end - end
        new_spans[i][1] += offset
        for new_span_entry in new_spans[i+1:]:
            new_span_entry[4] += offset

    # generate new text and create final spans
    new_text = replace_refspans(spans_to_replace, full_string, btwn_padding="")
    new_spans = [(start + offset, end + offset, token, surface) for start, end, token, surface, offset in new_spans]

    return new_text, new_spans





import dotenv
import hydra
import pyrootutils
from omegaconf import DictConfig

# load environment variables from `.env` file if it exists
# recursively searches for `.env` in all folders starting from work dir
dotenv.load_dotenv(override=True)

root = pyrootutils.find_root(search_from=__file__, indicator=["setup.py"])
root = root / "src"
root = pyrootutils.set_root(path=root, dotenv=True, pythonpath=True)

@hydra.main(version_base="1.2", config_path="configs/", config_name="train.yaml")
def main(config: DictConfig):

    from SciAssist import utils
    from SciAssist.pipelines.training_pipeline import train

    # Applies optional utilities
    utils.extras(config)

    # Train model
    return train(config)


if __name__ == "__main__":
    main()



from torch import nn
from transformers import PegasusForConditionalGeneration
from transformers.modeling_outputs import Seq2SeqLMOutput


class FrostForSummarization(nn.Module):
    def __init__(
        self,
        model_checkpoint: str = "pegasus/frost",
        cache_dir: str = ".cache",
        save_name: str = "frost-mup.pt",
        model_dir: str = "pretrained"
    ):
        super().__init__()
        self.save_name = save_name
        self.model_dir = model_dir
        self.pegasus = PegasusForConditionalGeneration.from_pretrained(
            model_checkpoint,
            cache_dir=cache_dir,
        )
        print(f"The model checkpoint are cached in '{cache_dir}'.")

    def forward(self, input_ids=None, attention_mask=None, labels=None):
        outputs = self.pegasus(input_ids, attention_mask=attention_mask, labels=labels)

        return Seq2SeqLMOutput(
            loss=outputs.loss,
            logits=outputs.logits
        )

    def generate(self, input_ids=None, attention_mask=None, num_beams=1, num_return_sequences=1):
        diversity_penalty = 0.0
        if num_return_sequences>1:
            diversity_penalty = 1.0
        return self.pegasus.generate(input_ids=input_ids, attention_mask=attention_mask,
                                  num_beams=num_beams,
                                  num_return_sequences=num_return_sequences,
                                  num_beam_groups=num_return_sequences,
                                  diversity_penalty=diversity_penalty)



# main developer: Yixi Ding <dingyixi@hotmail.com>
import math
import os
from pathlib import Path
from typing import Any, List

import bert_score
import numpy as np
import pandas as pd
import torch
from pytorch_lightning import LightningModule
from scipy.stats import pearsonr
from torchmetrics import MaxMetric
from torchmetrics.text.bert import BERTScore
from torchmetrics.text.rouge import ROUGEScore
from transformers import AutoModelForSeq2SeqLM

from SciAssist.utils.data_utils import DataUtilsForSeq2Seq


class MupBartLitModule(LightningModule):
    def __init__(
        self,
        model: AutoModelForSeq2SeqLM,
        data_utils = DataUtilsForSeq2Seq,
        lr: float = 2e-5,
    ):
        super().__init__()
        self.save_hyperparameters(logger=False)
        self.data_utils = data_utils
        self.model = model

        self.val_metric = ROUGEScore(use_stemmer=True)
        self.val_best_Rouge1 = MaxMetric()
        self.val_best_bertscore = MaxMetric()
        self.test_bertscore = BERTScore(use_stemmer=True)
        self.test_metric = ROUGEScore(use_stemmer=True)
        self.test_best_Rouge1 = MaxMetric()
        self.test_best_bertscore = MaxMetric()
        self.best_Rouge1 = 0
        self.best_bertscore = 0
        self.val_gen_lens = []
        self.val_gen_len = 0
        self.test_gen_lens = []
        self.test_gen_len = 0
        self.test_len_pred = []
        self.test_len_label = []
        self.test_preds = []
        self.test_labels = []
        self.save_path = Path(os.path.join(self.model.model_dir,self.model.save_name))

    def forward(self, inputs):
        return self.hparams.model(**inputs)


    def on_train_start(self):
        # self.val_bertscore.reset()
        self.val_metric.reset()
        self.val_best_Rouge1.reset()
        self.test_bertscore.reset()
        self.test_metric.reset()
        self.test_best_Rouge1.reset()

    def step(self, batch: Any):
        inputs, labels = batch, batch["labels"]
        outputs = self.forward(inputs)
        loss = outputs.loss
        return loss, labels

    def training_step(self, batch: Any, batch_idx: int):
        outputs = self.step(batch)
        loss = outputs[0]
        self.log("train/loss", loss, on_step=True, on_epoch=True, prog_bar=False)
        return {"loss": loss}

    def training_epoch_end(self, outputs: List[Any]):
        pass

    def validation_step(self, batch: Any, batch_idx: int):
        labels = batch["labels"]
        input_ids = batch["input_ids"]
        attention_mask = batch["attention_mask"]
        outputs = self.step(batch)
        loss = outputs[0]
        self.log("val/loss", loss, on_step=False, on_epoch=True, prog_bar=False)

        # Get prediction ids sequence
        preds = self.model.generate(input_ids=input_ids,attention_mask=attention_mask)
        # Convert ids to strings
        decoded_preds, decoded_labels = self.data_utils.postprocess(preds, labels)

        # Compute Rouge Metrics
        rouge_metric = self.val_metric(preds=decoded_preds, target=decoded_labels)

        result = {key: value * 100 for key, value in rouge_metric.items()}
        # decoded_preds = [decoded_pred.replace("\n", " ") for decoded_pred in decoded_preds]
        # decoded_labels = [decoded_label.replace("\n", " ") for decoded_label in decoded_labels]
        # bertscore_metric = self.val_bertscore(preds=decoded_preds, target=decoded_labels)

        # Compute average length of summaries
        self.val_gen_lens.extend([np.count_nonzero(pred != self.data_utils.tokenizer.pad_token_id) for pred in preds.to("cpu")])

        #  Log results


        self.log("val/Rouge-1", result["rouge1_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-1", result["rouge1_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-1", result["rouge1_recall"], on_step=False, on_epoch=True, prog_bar=True)

        self.log("val/Rouge-2", result["rouge2_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-2", result["rouge2_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-2", result["rouge2_recall"], on_step=False, on_epoch=True, prog_bar=True)

        self.log("val/Rouge-L", result["rougeL_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-L", result["rougeL_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-L", result["rougeL_recall"], on_step=False, on_epoch=True, prog_bar=True)

        self.log("val/Rouge-Lsum", result["rougeLsum_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/PRouge-Lsum", result["rougeLsum_precision"], on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/RRouge-Lsum", result["rougeLsum_recall"], on_step=False, on_epoch=True, prog_bar=True)

        return result

    def validation_epoch_end(self, outputs: List[Any]):
        rouge = self.val_metric.compute()
        # bert = self.val_bertscore.compute()
        # self.val_best_Rouge1.update(rouge["rouge1_fmeasure"])
        # # self.val_best_bertscore.update(bert["f1"])
        # # Save the best model
        # if rouge["rouge1_fmeasure"] > self.best_Rouge1:
        #     self.best_Rouge1 = rouge["rouge1_fmeasure"]
        #     print(f"Epoch: {self.current_epoch}: Save the current best model.")
        #     torch.save(self.model.state_dict(), self.save_path)

        self.val_best_Rouge1.update(rouge["rouge2_fmeasure"])
        # self.val_best_bertscore.update(bert["f1"])
        # Save the best model
        if rouge["rouge2_fmeasure"] > self.best_Rouge1:
            self.best_Rouge1 = rouge["rouge2_fmeasure"]
            print(f"Epoch: {self.current_epoch}: Save the current best model.")
            torch.save(self.model.state_dict(), self.save_path)

        # if numpy.mean(bert["f1"]) > self.best_bertscore:
        #     self.best_bertscore = numpy.mean(bert["f1"])
        #     print(f"Epoch: {self.current_epoch}: Save the current best model.")
        #     torch.save(self.model.state_dict(), self.save_path)


        # Compute the average length of summaries
        self.val_gen_len = np.mean(self.val_gen_lens)
        self.log("val/gen_len", self.val_gen_len, on_step=False, on_epoch=True, prog_bar=True)
        self.log("val/best_rouge1", self.val_best_Rouge1.compute(), on_epoch=True, prog_bar=True)

    def test_step(self, batch: Any, batch_idx: int):
        input_ids = batch["input_ids"]
        attention_mask = batch["attention_mask"]
        result = {}
        # torch.save(self.model.state_dict(), "/home/yixi/project/sciassist/src/pretrained/flant5_mup/negative.pt")
        # torch.save(self.model,"/home/dingyx/project/SciAssist/src/pretrained/flant5_mup/checkpoints/epoch_001.ckpt")
        # print("finished")

        if "labels" not in batch.keys():
            # Simply do prediction
            preds = self.model.generate(input_ids=input_ids, attention_mask=attention_mask)
            decoded_preds = self.data_utils.tokenizer.batch_decode(preds, skip_special_tokens=True)
            result["preds"] = decoded_preds

        else:
            # Do prediction and compute metrics
            # Get prediction ids sequence
            labels = batch["labels"]
            preds = self.model.generate(input_ids=input_ids, attention_mask=attention_mask)
            # Convert ids to strings
            decoded_preds, decoded_labels = self.data_utils.postprocess(preds, labels)

            # Compute Rouge Metrics
            rouge_metric = self.test_metric(preds=decoded_preds, target=decoded_labels)
            decoded_preds = [decoded_pred.replace("\n", " ") for decoded_pred in decoded_preds]
            decoded_labels = [decoded_label.replace("\n", " ") for decoded_label in decoded_labels]
            self.test_preds.extend(decoded_preds)
            self.test_labels.extend(decoded_labels)
            # print(bertscore_metric)

            # rounded_score = {k: numpy.mean([round(v, 3) for v in vv]) for k, vv in bertscore_metric.items()}
            # Compute average length of summaries
            #  Log results
            # self.log("test/BERTScore", rounded_score["f1"], on_step=False, on_epoch=True, prog_bar=True)

            result = {key: value * 100 for key, value in rouge_metric.items()}

            result['preds'] = decoded_preds
            result['id'] = batch['id']
            # Log results
            self.log("test/Rouge-1", result["rouge1_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            self.log("test/Rouge-2", result["rouge2_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            self.log("test/Rouge-L", result["rougeL_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            self.log("test/Rouge-Lsum", result["rougeLsum_fmeasure"], on_step=False, on_epoch=True, prog_bar=True)
            length_pred  = [len(pred.strip().split()) for pred in decoded_preds]
            length_label = [len(label.strip().split()) for label in decoded_labels]

            bucket_pred = [50*math.ceil(s/50) for s in length_pred]
            bucket_label = [50*math.ceil(s/50) for s in length_label]
            # bucket_label = batch["length"].to("cpu").tolist()


            # for a,b in zip(decoded_preds,decoded_labels):
            #     print(len(a.strip().split()),a)
            #     print("-----")
            #     print(len(b.strip().split()),b)
            #     print("=========")
            # MAD = np.mean([np.absolute(pred-label) for pred,label in zip(bucket_pred,bucket_label)])
            # length_acc = np.mean([ pred<=label for pred,label in zip(bucket_pred, bucket_label)])
            # length_acc2 = np.mean([pred == label for pred, label in zip(bucket_pred, bucket_label)])
            self.test_len_pred.extend(bucket_pred)
            self.test_len_label.extend(bucket_label)

        # Compute average length of summaries
        self.test_gen_lens.extend([np.count_nonzero(pred != self.data_utils.tokenizer.pad_token_id) for pred in preds.to("cpu")])
        return result


    def test_epoch_end(self, outputs: List[Any]):
        # Save prediction results
        # with open(os.path.join(self.model.model_dir,"prediction.txt"),'w') as f:
        #     for batch in outputs:
        #         for res in batch["preds"]:
        #             f.write(res)
        #             f.write("\n")

        for batch in outputs:
            for id,res in zip(batch['id'],batch["preds"]):
                with open("/home/dingyx/project/SciAssist/data/pdfs/summary_flant5/"  + str(id.item()) +".txt","a") as f:
                    # print("/home/dingyx/project/SciAssist/data/MUP_CTRLkeyword/" + str(id.item()) +".txt")
                    f.write(res)
                    f.write("\n")
                    # f.write(str(len(res.split(" "))))

        P,R,F1 = bert_score.score(self.test_preds, self.test_labels,
                                            rescale_with_baseline=True, lang="en")
        # Compute average length of summaries
        self.test_gen_len = np.mean(self.test_gen_lens)
        bucket_pred, bucket_label = self.test_len_pred, self.test_len_label
        bucket = {"bucket_pred": bucket_pred, "bucket_lable": bucket_label}
        bucket = pd.DataFrame(bucket)
        # bucket.to_csv("/home/dingyx/project/SciAssist/data/FLANT5_len/instances.csv", index=True)

        pcc = pearsonr(self.test_len_label, self.test_len_pred)
        PCC, pvalue = pcc[0], pcc[1]
        print(pcc)
        MAD = np.mean([np.absolute(pred - label) for pred, label in zip(bucket_pred, bucket_label)])
        length_acc = np.mean([pred <= label for pred, label in zip(bucket_pred, bucket_label)])
        length_acc2 = np.mean([pred == label for pred, label in zip(bucket_pred, bucket_label)])

        self.log("MAD", MAD, on_step=False, on_epoch=True, prog_bar=True)
        self.log("length_acc", length_acc, on_step=False, on_epoch=True, prog_bar=True)
        self.log("length_acc2", length_acc2, on_step=False, on_epoch=True, prog_bar=True)
        self.log("BertScore", F1.mean(), on_step=False, on_epoch=True, prog_bar=True)
        self.log("PCC", PCC, on_step=False, on_epoch=True, prog_bar=True)
        self.log("P-Value", pvalue, on_step=False, on_epoch=True, prog_bar=True)
        self.log("test/gen_len", self.test_gen_len, on_step=False, on_epoch=True, prog_bar=True)



    def on_epoch_end(self):
        self.val_metric.reset()
        # self.val_bertscore.reset()
        self.val_best_Rouge1.reset()
        self.val_gen_lens = []
        self.test_metric.reset()
        self.test_bertscore.reset()
        self.test_best_Rouge1.reset()
        self.test_gen_lens = []

    def configure_optimizers(self):
        optimizer = torch.optim.AdamW(
            params=self.hparams.model.parameters(), lr=self.hparams.lr
        )
        lr_scheduler = torch.optim.lr_scheduler.StepLR(optimizer, step_size=1)
        return [optimizer],[lr_scheduler]



import nltk
from sklearn.decomposition import TruncatedSVD
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.metrics.pairwise import cosine_similarity
import numpy as np
vectorizer = TfidfVectorizer(stop_words='english')

# def extract_related_sentences(text, keyword=None, num_sentences=3):
#
#     if text.strip()=="":
#         return text
#     if keyword==None:
#         return text
#     # tokenize sentences and convert to lowercase
#     origin_sentences = nltk.sent_tokenize(text)
#     sentences = [sentence.lower() for sentence in origin_sentences]
#
#     # create TF-IDF vectorizer
#
#     # fit and transform the sentences into TF-IDF vectors
#     tfidf_vectors = vectorizer.fit_transform(sentences)
#
#     # compute cosine similarity between the keyword and each sentence
#     keyword_vector = vectorizer.transform([keyword.lower()])
#     cos_similarities = cosine_similarity(tfidf_vectors, keyword_vector).flatten()
#
#     # extract top 3 sentences with highest cosine similarity to the keyword
#     related_sentences = [origin_sentences[i] for i in cos_similarities.argsort()[::-1][:num_sentences]]
#     related_sentences = " ".join(related_sentences)
#     print(related_sentences)
#     return related_sentences


import nltk
from nltk.tokenize import word_tokenize, sent_tokenize
from nltk.corpus import stopwords
from nltk.stem import WordNetLemmatizer
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.decomposition import TruncatedSVD

from sklearn.metrics.pairwise import cosine_similarity


def extract_related_sentences2(text,keyword,related_words_num=2):

    # nltk.download('wordnet')

    # Tokenize the text and remove stop words
    stop_words = set(stopwords.words('english'))
    sentences = text.split(".")
    lemmatizer = WordNetLemmatizer()

    corpus = []
    for sentence in sentences:
        words = word_tokenize(sentence)
        words = [lemmatizer.lemmatize(word.lower()) for word in words if
                 word.isalpha() and word.lower() not in stop_words]
        corpus.append(" ".join(words))

    # Extract related words

    # Define the corpus of text and the keyword
    processed_corpus = []
    for document in corpus:
        words = nltk.word_tokenize(document.lower())
        words = [lemmatizer.lemmatize(word) for word in words if word.isalpha() and word not in stop_words]
        processed_sents = " ".join(words).replace(keyword, keyword.replace(" ", "_"))
        processed_corpus.append(processed_sents)

    # Create a TF-IDF matrix
    vectorizer = TfidfVectorizer(max_features=1000)

    tfidf_matrix = vectorizer.fit_transform(processed_corpus)
    # Perform LSA
    lsa = TruncatedSVD(n_components=100, random_state=42)
    lsa_matrix = lsa.fit_transform(tfidf_matrix)

    # Identify related terms
    term_matrix = vectorizer.transform([keyword.replace(" ", "_")])
    term_vector = lsa.transform(term_matrix)

    similarities = cosine_similarity(term_vector, lsa_matrix)
    related_indices = similarities.argsort()[0][:related_words_num]

    related_terms = [vectorizer.get_feature_names_out()[i] for i in related_indices]

    print("Related terms for '{}': {}".format(keyword, ", ".join(related_terms)))

    # Get the indices of the sentences containing the keywords
    keyword_indices = []
    keywords = [keyword] + related_terms
    for i, sentence in enumerate(sentences):
        for keyword in keywords:
            if keyword.lower() in sentence.lower():
                keyword_indices.append(i)

    # Extract the relevant content using LSA
    relevant_content = []
    indexes = []
    for i in keyword_indices:
        sentence = sentences[i]
        topic_vector = lsa_matrix[i]
        similarities = lsa_matrix.dot(topic_vector)
        similar_indices = similarities.argsort()[::-1][1:]
        top_sentence_index = similar_indices[0]
        if top_sentence_index not in indexes:
            top_sentence = sentences[top_sentence_index]
            relevant_content.append(top_sentence)
            indexes.append(top_sentence_index)
    return " . ".join(relevant_content) + " ."


import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer

def extract_related_sentences(text, keyword=None, num_sentences=5):

    if keyword==None:
        return text
    # Split the text into sentences
    sentences = text.split(".")
    # Remove any leading/trailing white space from each sentence
    sentences = [s.strip() for s in sentences]
    # Initialize a TfidfVectorizer
    vectorizer = TfidfVectorizer(stop_words='english')
    # Create a matrix of TF-IDF scores for each sentence
    sentence_matrix = vectorizer.fit_transform(sentences)
    # Get the index of the keyword in the feature vocabulary
    keyword_vector = vectorizer.transform([keyword.lower()])
    cos_similarities = cosine_similarity(sentence_matrix, keyword_vector).flatten()

    # extract top 3 sentences with highest cosine similarity to the keyword
    related_sentences = [sentences[i] for i in cos_similarities.argsort()[::-1][:num_sentences]]


    # Return the top N related sentences
    return " . ".join(related_sentences) + " ."



LABEL_NAMES = [
    "author",
    "booktitle",
    "date",
    "editor",
    "institution",
    "journal",
    "location",
    "note",
    "pages",
    "publisher",
    "tech",
    "title",
    "volume"
]

num_labels = len(LABEL_NAMES)

id2label = {str(i): label for i, label in enumerate(LABEL_NAMES)}
label2id = {v: k for k, v in id2label.items()}



import os
import platform
from subprocess import run

PACKAGE_ROOT = os.path.dirname(__file__)
SCRIPTS_DIR = {'grobid': os.path.join(PACKAGE_ROOT, 'doc2json/scripts/')}

def setup_grobid():
    if platform.system().lower()=="linux":
        script = os.path.join(SCRIPTS_DIR['grobid'], 'setup_grobid_linux.sh')
        run(['bash',script])
    print("Grobid is installed.")


def run_grobid():
    if platform.system().lower()=="linux":
        script = os.path.join(SCRIPTS_DIR['grobid'], 'run_grobid_linux.sh')
        cmd = "nohup bash " + script + " >/dev/null 2>grobid_log &"
        run(cmd, shell=True)
    print("Grobid is running now.")




import json

import pytest

from SciAssist import ReferenceStringParsing


# Rsp
@pytest.fixture
def BERTRspGrobid():
    res = []
    with open("output/grobid/BERT_paper_rsp_scibert.json", "r", encoding="utf-8") as f:
        results = f.readlines()
        for i in results:
            r = i.replace("\n","")
            res.append(json.loads(r))
    return res


@pytest.fixture
def BERTRspPDFMiner():
    res = []
    with open("output/pdfminer-six/BERT_paper_rsp_scibert.json", "r", encoding="utf-8") as f:
        results = f.readlines()
        for i in results:
            r = i.replace("\n","")
            res.append(json.loads(r))
    return res

class TestRsp():

    def test_RspGrobid(self, BERTRspGrobid):

        parser = ReferenceStringParsing()
        res = parser.predict("pdfs/BERT_paper.pdf")
        assert res == BERTRspGrobid


    def test_RspPDFMiner(self, BERTRspPDFMiner):
        parser = ReferenceStringParsing(os_name="nt")
        res = parser.predict("pdfs/BERT_paper.pdf")
        assert res == BERTRspPDFMiner





from pathlib import Path
from typing import Optional

import datasets
from datasets import Dataset, DatasetDict
from pytorch_lightning import LightningDataModule
from torch.utils.data import DataLoader

from SciAssist.utils.data_reader import csv_reader
from SciAssist.utils.data_utils import DataUtilsForDatasetExtraction


class DatasetExtractionModule(LightningDataModule):
    def __init__(
        self,
        data_repo: str,
        train_batch_size: int = 32,
        num_workers: int = 0,
        pin_memory: bool = False,
        data_cache_dir: str = ".cache",
        seed: int = 176,
        data_utils = DataUtilsForDatasetExtraction
    ):
        super().__init__()
        self.save_hyperparameters(logger=False)
        self.data_cache_dir = Path(self.hparams.data_cache_dir)
        self.data_utils = self.hparams.data_utils

        self.data_train: Optional[Dataset] = None
        self.data_val: Optional[Dataset] = None
        self.data_test: Optional[Dataset] = None


    def setup(self, stage: Optional[str] = None):
        if not self.data_train and not self.data_val and not self.data_test:
            data_files = {"data": "sentences.txt", "labels": "labs.txt", "tags": "tags.txt"}
            print(self.hparams.data_repo + '/train/')
            raw_train_dataset = datasets.load_dataset(
                path='text',
                data_dir=self.hparams.data_repo + '/train/',
                data_files=data_files,
                cache_dir=self.data_cache_dir
            )
            raw_val_dataset = datasets.load_dataset(
                path='text',
                data_dir=self.hparams.data_repo + '/val/',
                data_files=data_files,
                cache_dir=self.data_cache_dir
            )
            raw_test_dataset = datasets.load_dataset(
                path='text',
                data_dir=self.hparams.data_repo + '/test/',
                data_files=data_files,
                cache_dir=self.data_cache_dir
            )

            self.data_train = self.data_utils.tokenize_and_align_labels(raw_train_dataset)
            self.data_val = self.data_utils.tokenize_and_align_labels(raw_val_dataset)
            self.data_test = self.data_utils.tokenize_and_align_labels(raw_test_dataset)


    def train_dataloader(self):
        return DataLoader(
            dataset=self.data_train,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_train.collate_fn,
            shuffle=True,
        )

    def val_dataloader(self):
        return DataLoader(
            dataset=self.data_val,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_val.collate_fn,
            shuffle=False,
        )

    def test_dataloader(self):
        return DataLoader(
            dataset=self.data_test,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_test.collate_fn,
            shuffle=False,
        )




# utility functions for handling failure situations with grobid-detected citation spans

import re
from typing import Dict, List, Tuple


BRACKET_REGEX = re.compile(r'\[[1-9]\d{0,2}([,;\-\s]+[1-9]\d{0,2})*;?\]')
BRACKET_STYLE_THRESHOLD = 5

SINGLE_BRACKET_REGEX = re.compile(r'\[([1-9]\d{0,2})\]')
EXPANSION_CHARS = {'-', '–'}


def span_already_added(sub_start: int, sub_end: int, span_indices: List[Tuple[int, int]]) -> bool:
    """
    Check if span is a subspan of existing span
    :param sub_start:
    :param sub_end:
    :param span_indices:
    :return:
    """
    for span_start, span_end in span_indices:
        if sub_start >= span_start and sub_end <= span_end:
            return True
    return False


def is_expansion_string(between_string: str) -> bool:
    """
    Check if the string between two refs is an expansion string
    :param between_string:
    :return:
    """
    if len(between_string) <= 2 \
            and any([c in EXPANSION_CHARS for c in between_string]) \
            and all([c in EXPANSION_CHARS.union({' '}) for c in between_string]):
        return True
    return False


# TODO: still cases like `09bcee03baceb509d4fcf736fa1322cb8adf507f` w/ dups like ['L Jung', 'R Hessler', 'Louis Jung', 'Roland Hessler']
# example paper that has empties & duplicates: `09bce26cc7e825e15a4469e3e78b7a54898bb97f`
def _clean_empty_and_duplicate_authors_from_grobid_parse(authors: List[Dict]) -> List[Dict]:
    """
    Within affiliation, `location` is a dict with fields <settlement>, <region>, <country>, <postCode>, etc.
    Too much hassle, so just take the first one that's not empty.
    """
    # stripping empties
    clean_authors_list = []
    for author in authors:
        clean_first = author['first'].strip()
        clean_last = author['last'].strip()
        clean_middle = [m.strip() for m in author['middle']]
        clean_suffix = author['suffix'].strip()
        if clean_first or clean_last or clean_middle:
            author['first'] = clean_first
            author['last'] = clean_last
            author['middle'] = clean_middle
            author['suffix'] = clean_suffix
            clean_authors_list.append(author)
    # combining duplicates (preserve first occurrence of author name as position)
    key_to_author_blobs = {}
    ordered_keys_by_author_pos = []
    for author in clean_authors_list:
        key = (author['first'], author['last'], ' '.join(author['middle']), author['suffix'])
        if key not in key_to_author_blobs:
            key_to_author_blobs[key] = author
            ordered_keys_by_author_pos.append(key)
        else:
            if author['email']:
                key_to_author_blobs[key]['email'] = author['email']
            if author['affiliation'] and (author['affiliation']['institution'] or author['affiliation']['laboratory'] or author['affiliation']['location']):
                key_to_author_blobs[key]['affiliation'] = author['affiliation']
    dedup_authors_list = [key_to_author_blobs[key] for key in ordered_keys_by_author_pos]
    return dedup_authors_list


import re
from collections import defaultdict
from typing import List, Dict, Optional

import bs4
from bs4 import BeautifulSoup

SUBSTITUTE_TAGS = {
    'persName',
    'orgName',
    'publicationStmt',
    'titleStmt',
    'biblScope'
}


def clean_tags(el: bs4.element.Tag):
    """
    Replace all tags with lowercase version
    :param el:
    :return:
    """
    for sub_tag in SUBSTITUTE_TAGS:
        for sub_el in el.find_all(sub_tag):
            sub_el.name = sub_tag.lower()


def soup_from_path(file_path: str):
    """
    Read XML file
    :param file_path:
    :return:
    """
    return BeautifulSoup(open(file_path, "rb").read(), "xml")


def get_title_from_grobid_xml(raw_xml: BeautifulSoup) -> str:
    """
    Returns title
    :return:
    """
    for title_entry in raw_xml.find_all("title"):
        if title_entry.has_attr("level") \
                and title_entry["level"] == "a":
            return title_entry.text
    try:
        return raw_xml.title.text
    except AttributeError:
        return ""


def get_author_names_from_grobid_xml(raw_xml: BeautifulSoup) -> List[Dict[str, str]]:
    """
    Returns a list of dictionaries, one for each author,
    containing the first and last names.

    e.g.
        {
            "first": first,
            "middle": middle,
            "last": last,
            "suffix": suffix
        }
    """
    names = []

    for author in raw_xml.find_all("author"):
        if not author.persname:
            continue

        # forenames include first and middle names
        forenames = author.persname.find_all("forename")

        # surnames include last names
        surnames = author.persname.find_all("surname")

        # name suffixes
        suffixes = author.persname.find_all("suffix")

        first = ""
        middle = []
        last = ""
        suffix = ""

        for forename in forenames:
            if forename["type"] == "first":
                if not first:
                    first = forename.text
                else:
                    middle.append(forename.text)
            elif forename["type"] == "middle":
                middle.append(forename.text)

        if len(surnames) > 1:
            for surname in surnames[:-1]:
                middle.append(surname.text)
            last = surnames[-1].text
        elif len(surnames) == 1:
            last = surnames[0].text

        if len(suffix) >= 1:
            suffix = " ".join([suffix.text for suffix in suffixes])

        names_dict = {
            "first": first,
            "middle": middle,
            "last": last,
            "suffix": suffix
        }

        names.append(names_dict)
    return names


def get_affiliation_from_grobid_xml(raw_xml: BeautifulSoup) -> Dict:
    """
    Get affiliation from grobid xml
    :param raw_xml:
    :return:
    """
    location_dict = dict()
    laboratory_name = ""
    institution_name = ""

    if raw_xml and raw_xml.affiliation:
        for child in raw_xml.affiliation:
            if child.name == "orgname":
                if child.has_attr("type"):
                    if child["type"] == "laboratory":
                        laboratory_name = child.text
                    elif child["type"] == "institution":
                        institution_name = child.text
            elif child.name == "address":
                for grandchild in child:
                    if grandchild.name and grandchild.text:
                        location_dict[grandchild.name] = grandchild.text

        if laboratory_name or institution_name:
            return {
                "laboratory": laboratory_name,
                "institution": institution_name,
                "location": location_dict
            }

    return {}


def get_author_data_from_grobid_xml(raw_xml: BeautifulSoup) -> List[Dict]:
    """
    Returns a list of dictionaries, one for each author,
    containing the first and last names.

    e.g.
        {
            "first": first,
            "middle": middle,
            "last": last,
            "suffix": suffix,
            "affiliation": {
                "laboratory": "",
                "institution": "",
                "location": "",
            },
            "email": ""
        }
    """
    authors = []

    for author in raw_xml.find_all("author"):

        first = ""
        middle = []
        last = ""
        suffix = ""

        if author.persname:
            # forenames include first and middle names
            forenames = author.persname.find_all("forename")

            # surnames include last names
            surnames = author.persname.find_all("surname")

            # name suffixes
            suffixes = author.persname.find_all("suffix")

            for forename in forenames:
                if forename.has_attr("type"):
                    if forename["type"] == "first":
                        if not first:
                            first = forename.text
                        else:
                            middle.append(forename.text)
                    elif forename["type"] == "middle":
                        middle.append(forename.text)

            if len(surnames) > 1:
                for surname in surnames[:-1]:
                    middle.append(surname.text)
                last = surnames[-1].text
            elif len(surnames) == 1:
                last = surnames[0].text

            if len(suffix) >= 1:
                suffix = " ".join([suffix.text for suffix in suffixes])

        affiliation = get_affiliation_from_grobid_xml(author)

        email = ""
        if author.email:
            email = author.email.text

        author_dict = {
            "first": first,
            "middle": middle,
            "last": last,
            "suffix": suffix,
            "affiliation": affiliation,
            "email": email
        }

        authors.append(author_dict)

    return authors


def get_year_from_grobid_xml(raw_xml: BeautifulSoup) -> Optional[int]:
    """
    Returns date published if exists
    :return:
    """
    if raw_xml.date and raw_xml.date.has_attr("when"):
        # match year in date text (which is in some unspecified date format)
        year_match = re.match(r"((19|20)\d{2})", raw_xml.date["when"])
        if year_match:
            year = year_match.group(0)
            if year and year.isnumeric() and len(year) == 4:
                return int(year)
    return None


def get_venue_from_grobid_xml(raw_xml: BeautifulSoup, title_text: str) -> str:
    """
    Returns venue/journal/publisher of bib entry
    Grobid ref documentation: https://grobid.readthedocs.io/en/latest/training/Bibliographical-references/
    level="j": journal title
    level="m": "non journal bibliographical item holding the cited article"
    level="s": series title
    :return:
    """
    title_names = []
    keep_types = ["j", "m", "s"]
    # get all titles of the anove types
    for title_entry in raw_xml.find_all("title"):
        if title_entry.has_attr("level") \
                and title_entry["level"] in keep_types \
                and title_entry.text != title_text:
            title_names.append((title_entry["level"], title_entry.text))
    # return the title name that most likely belongs to the journal or publication venue
    if title_names:
        title_names.sort(key=lambda x: keep_types.index(x[0]))
        return title_names[0][1]
    return ""


def get_volume_from_grobid_xml(raw_xml: BeautifulSoup) -> str:
    """
    Returns the volume number of grobid bib entry
    Grobid <biblscope unit="volume">
    :return:
    """
    for bibl_entry in raw_xml.find_all("biblscope"):
        if bibl_entry.has_attr("unit") and bibl_entry["unit"] == "volume":
            return bibl_entry.text
    return ""


def get_issue_from_grobid_xml(raw_xml: BeautifulSoup) -> str:
    """
    Returns the issue number of grobid bib entry
    Grobid <biblscope unit="issue">
    :return:
    """
    for bibl_entry in raw_xml.find_all("biblscope"):
        if bibl_entry.has_attr("unit") and bibl_entry["unit"] == "issue":
            return bibl_entry.text
    return ""


def get_pages_from_grobid_xml(raw_xml: BeautifulSoup) -> str:
    """
    Returns the page numbers of grobid bib entry
    Grobid <biblscope unit="page">
    :return:
    """
    for bibl_entry in raw_xml.find_all("biblscope"):
        if bibl_entry.has_attr("unit") and bibl_entry["unit"] == "page" and bibl_entry.has_attr("from"):
            from_page = bibl_entry["from"]
            if bibl_entry.has_attr("to"):
                to_page = bibl_entry["to"]
                return f'{from_page}--{to_page}'
            else:
                return from_page
    return ""


def get_other_ids_from_grobid_xml(raw_xml: BeautifulSoup) -> Dict[str, List]:
    """
    Returns a dictionary of other identifiers from grobid bib entry (arxiv, pubmed, doi)
    :param raw_xml:
    :return:
    """
    other_ids = defaultdict(list)

    for idno_entry in raw_xml.find_all("idno"):
        if idno_entry.has_attr("type") and idno_entry.text:
            other_ids[idno_entry["type"]].append(idno_entry.text)

    return other_ids


def get_raw_bib_text_from_grobid_xml(raw_xml: BeautifulSoup) -> str:
    """
    Returns the raw bibiliography string
    :param raw_xml:
    :return:
    """
    for note in raw_xml.find_all("note"):
        if note.has_attr("type") and note["type"] == "raw_reference":
            return note.text
    return ""


def get_publication_datetime_from_grobid_xml(raw_xml: BeautifulSoup) -> str:
    """
    Finds and returns the publication datetime if it exists
    :param raw_xml:
    :return:
    """
    if raw_xml.publicationStmt:
        for child in raw_xml.publicationstmt:
            if child.name == "date" \
                    and child.has_attr("type") \
                    and child["type"] == "published" \
                    and child.has_attr("when"):
                return child["when"]
    return ""


def parse_bib_entry(bib_entry: BeautifulSoup) -> Dict:
    """
    Parse one bib entry
    :param bib_entry:
    :return:
    """
    clean_tags(bib_entry)
    title = get_title_from_grobid_xml(bib_entry)
    return {
        'ref_id': bib_entry.attrs.get("xml:id", None),
        'title': title,
        'authors': get_author_names_from_grobid_xml(bib_entry),
        'year': get_year_from_grobid_xml(bib_entry),
        'venue': get_venue_from_grobid_xml(bib_entry, title),
        'volume': get_volume_from_grobid_xml(bib_entry),
        'issue': get_issue_from_grobid_xml(bib_entry),
        'pages': get_pages_from_grobid_xml(bib_entry),
        'other_ids': get_other_ids_from_grobid_xml(bib_entry),
        'raw_text': get_raw_bib_text_from_grobid_xml(bib_entry),
        'urls': []
    }


def is_reference_tag(tag: bs4.element.Tag) -> bool:
    return tag.name == "ref" and tag.attrs.get("type", "") == "bibr"


def extract_paper_metadata_from_grobid_xml(tag: bs4.element.Tag) -> Dict:
    """
    Extract paper metadata (title, authors, affiliation, year) from grobid xml
    :param tag:
    :return:
    """
    clean_tags(tag)
    paper_metadata = {
        "title": tag.titlestmt.title.text,
        "authors": get_author_data_from_grobid_xml(tag),
        "year": get_publication_datetime_from_grobid_xml(tag)
    }
    return paper_metadata


import xml.etree.ElementTree as ET
import os
file_list = []

root_dir = "/home/yixi/project/scisumm-corpus/data/Training-Set-2019/Task2/From-ScisummNet-2019"
for dirpath,dirnames,files in os.walk(root_dir):
    file_list = dirnames
    break
filter_files=[]

def parse_rec(filename):
    file = os.path.join(root_dir,filename,"Reference_XML",filename+".xml")
    tree = ET.parse(file)  # 解析读取xml函数


    # Element.findall()查找当前元素的直接子元素中带有指定标签的元素
    # Element.find()找带有特定标签的第一个子级
    # Elemtn.text 访问元素的文本内容
    # Element.get 访问元素的属性。
    body_text = ""

    if len(tree.findall('SECTION'))>0:
        for section in tree.findall('SECTION'):
            if "Introduction" in section.get("title") or "Conclusion" in section.get("title") or "1" in section.get("title"):
                for s in section.findall("S"):
                    body_text += s.text
                    body_text += " "
    else:
        filter_files.append(filename)
    # else:
        # for s in tree.findall("S"):
        #     if s.text is not None:
        #         body_text += s.text
        #         body_text += " "
        # print(filename)

    with open(os.path.join(root_dir,filename,filename+".txt"),"w") as f:
        f.write(body_text)

    return body_text

for file in file_list:
    s = parse_rec(file)

print("Finished.")
print(filter_files)



"""
Many of the REGEX expressions and pipeline in this set of utilities are borrowed or extended from
the unarXive project: https://github.com/IllDepence/unarXive

Modifications have been made to better identify the primary latex file and expand all other latex
files into the main file. Latexpand and tralics options have also been changed.
"""
import glob
import os
import re
import subprocess
import tempfile

import chardet
import magic

MAIN_TEX_PATT = re.compile(r'(\\begin\s*\{\s*document\s*\})', re.I)
# ^ with capturing parentheses so that the pattern can be used for splitting
PDF_EXT_PATT = re.compile(r'^\.pdf$', re.I)
GZ_EXT_PATT = re.compile(r'^\.gz$', re.I)
TEX_EXT_PATT = re.compile(r'^\.tex$', re.I)
NON_TEXT_PATT = re.compile(r'^\.(pdf|eps|jpg|png|gif)$', re.I)
BBL_SIGN = '\\bibitem'
# natbib fix
PRE_FIX_NATBIB = True
NATBIB_PATT = re.compile((r'\\cite(t|p|alt|alp|author|year|yearpar)\s*?\*?\s*?'
                           '(\[[^\]]*?\]\s*?)*?\s*?\*?\s*?\{([^\}]+?)\}'),
                         re.I)
# bibitem option fix
PRE_FIX_BIBOPT = True
BIBOPT_PATT = re.compile(r'\\bibitem\s*?\[[^]]*?\]', re.I|re.M)

# ↑ above two solve most tralics problems; except for mnras style bibitems
# (https://ctan.org/pkg/mnras)

# agressive math pre-removal
PRE_FILTER_MATH = False
FILTER_PATTS = []
for env in ['equation', 'displaymath', 'array', 'eqnarray', 'align', 'gather',
            'multline', 'flalign', 'alignat']:
    s = r'\\begin\{{{0}[*]?\}}.+?\\end\{{{0}\}}'.format(env)
    patt = re.compile(s, re.I | re.M | re.S)
    FILTER_PATTS.append(patt)
FILTER_PATTS.append(re.compile(r'\$\$.+?\$\$', re.S))
FILTER_PATTS.append(re.compile(r'\$.+?\$', re.S))
FILTER_PATTS.append(re.compile(r'\\\(.+?\\\)', re.S))
FILTER_PATTS.append(re.compile(r'\\\[.+?\\\]', re.S))


def read_file(path):
    try:
        with open(path) as f:
            cntnt = f.read()
    except UnicodeDecodeError:
        blob = open(path, 'rb').read()
        m = magic.Magic(mime_encoding=True)
        encoding = m.from_buffer(blob)
        try:
            cntnt = blob.decode(encoding)
        except (UnicodeDecodeError, LookupError) as e:
            encoding = chardet.detect(blob)['encoding']
            if encoding:
                try:
                    cntnt = blob.decode(encoding, errors='replace')
                except:
                    return ''
            else:
                return ''
    return cntnt


def remove_math(latex_str):
    parts = re.split(MAIN_TEX_PATT, latex_str, maxsplit=1)
    for patt in FILTER_PATTS:
         parts[2] = re.sub(patt, '', parts[2])
    return ''.join(parts)


def normalize(path, out_dir, write_logs=True):
    """
    Normalize an arXiv file
    Adapted from https://github.com/IllDepence/unarXive
        with modifications

    Identifies the primary *.tex file, the bibliography file,
    and expands other tex files and the bibliography into the
    main tex file
    """
    def log(msg):
        if write_logs:
            with open(os.path.join(out_dir, 'log.txt'), 'a') as f:
                f.write('{}\n'.format(msg))

    # break path
    _, fn = os.path.split(path.strip('/'))

    # identify main tex file
    main_tex_path = None
    ignored_names = []

    # check .tex files first
    for tfn in os.listdir(path):

        if not TEX_EXT_PATT.match(os.path.splitext(tfn)[1]):
            ignored_names.append(tfn)
            continue

        try:
            cntnt = read_file(os.path.join(path, tfn))
        except:
            continue

        if re.search(MAIN_TEX_PATT, cntnt) is not None:
            main_tex_path = tfn

    # try other files
    if main_tex_path is None:
        for tfn in ignored_names:
            if NON_TEXT_PATT.match(os.path.splitext(tfn)[1]):
                continue
            try:
                cntnt = read_file(os.path.join(path, tfn))
                if re.search(MAIN_TEX_PATT, cntnt) is not None:
                    main_tex_path = tfn
            except:
                continue

    # give up
    if main_tex_path is None:
        log(('couldn\'t find main tex file in dump archive {}'
             '').format(fn))

    # flatten to single tex file and save
    with tempfile.TemporaryDirectory() as tmp_dir_path:
        temp_tex_fn = os.path.join(tmp_dir_path, f'{fn}.tex')

        # find bbl file
        main_tex_fn = os.path.join(path, main_tex_path)
        bbl_files = glob.glob(os.path.join(path, '*.bbl'))

        if bbl_files:
            latexpand_args = ['latexpand',
                              '--expand-bbl',
                              os.path.split(bbl_files[0])[1],
                              main_tex_path,
                              '--output',
                              temp_tex_fn]
        else:
            latexpand_args = ['latexpand',
                              main_tex_path,
                              '--output',
                              temp_tex_fn]

        # run latexpand
        with open(os.path.join(out_dir, 'log_latexpand.txt'), 'a+') as err:
            subprocess.run(latexpand_args, stderr=err, cwd=path)

        # re-read and write to ensure utf-8 b/c latexpand doesn't
        # behave
        new_tex_fn = os.path.join(out_dir, f'{fn}.tex')
        cntnt = read_file(temp_tex_fn)
        if PRE_FIX_NATBIB:
            cntnt = NATBIB_PATT.sub(r'\\cite{\3}', cntnt)
        if PRE_FIX_BIBOPT:
            cntnt = BIBOPT_PATT.sub(r'\\bibitem', cntnt)
        if PRE_FILTER_MATH:
            cntnt = remove_math(cntnt)
        with open(new_tex_fn, mode='w', encoding='utf-8') as f:
            f.write(cntnt)


def latex_to_xml(tex_file: str, out_dir: str, out_file: str, err_file: str, log_file: str):
    """
    Convert expanded latex file to XML using tralics
    :param tex_file:
    :param out_dir:
    :param out_file:
    :param err_file:
    :param log_file:
    :return:
    """
    with open(os.devnull, 'w') as devnull, \
            open(err_file, 'a+') as err_f, \
            open(log_file, 'a+') as skip_f:
        # run tralics
        tralics_args = ['tralics',
                        '-silent',
                        '-noxmlerror',
                        '-utf8',
                        '-oe8',
                        '-entnames=false',
                        '-nomathml',
                        f'-output_dir={out_dir}',
                        tex_file]
        try:
            subprocess.run(tralics_args, stdout=devnull, stderr=err_f, timeout=5)
        except subprocess.TimeoutExpired:
            skip_f.write(f'{tex_file}\n')

        # if no output, skip
        if not os.path.exists(out_file):
            skip_f.write(f'{tex_file}\n')

    if os.path.exists(out_file):
        return out_file



# main developer: Yixi Ding <dingyixi@hotmail.com>
import json
import math
import os
from pathlib import Path
from typing import Optional

import datasets
import torch
from datasets import Dataset, DatasetDict
from pytorch_lightning import LightningDataModule
from torch.utils.data import DataLoader, random_split

from SciAssist.utils.data_utils import DataUtilsForSeq2Seq, DataUtilsForFlanT5


class LongSummDataModule(LightningDataModule):
    def __init__(
        self,
        train_batch_size: int = 8,
        num_workers: int = 0,
        pin_memory: bool = False,
        data_cache_dir: str = ".cache",
        seed: int = 777,
        data_utils = DataUtilsForFlanT5
    ):
        super().__init__()
        self.save_hyperparameters(logger=False)
        self.data_cache_dir = Path(self.hparams.data_cache_dir)
        self.data_utils = self.hparams.data_utils
        self.data_collator = self.data_utils.collator()

        self.data_train: Optional[Dataset] = None
        self.data_val: Optional[Dataset] = None
        self.data_test: Optional[Dataset] = None

    def prepare_data(self) -> DatasetDict:
        texts = []
        summaries = []
        lengths = []
        text_dir = "/home/dingyx/project/SciAssist/data/longsumm/"
        for (root,dirs,files) in os.walk("/home/dingyx/project/SciAssist/data/LongSumm-master/abstractive_summaries/by_clusters/", topdown=False):
            if len(files)<1:
                break
            for file in files:
                try:
                    json_file = os.path.join(root,file)
                    with open(json_file, 'r') as f:
                        data = json.load(f)
                        id = str(data["id"]) + ".txt"
                        text_file = os.path.join(text_dir, id)
                        with open(text_file, 'r') as f:
                            text = f.readlines()
                            text = " ".join(text)
                            texts.append(text)
                        summary = " ".join(data["summary"])
                        summaries.append(summary)
                except:
                    print("File not exsists.")

        raw_datasets = {
            "text": texts,
            "summary": summaries,
        }
        raw_datasets = Dataset.from_dict(raw_datasets)

        return raw_datasets

    def setup(self, stage: Optional[str] = None):
        if not self.data_train and not self.data_val and not self.data_test:
            raw_datasets = self.prepare_data()
            processed_datasets = DatasetDict()
            processed_datasets["whole"] = raw_datasets
            tokenized_datasets = processed_datasets.map(
                lambda x: self.data_utils.tokenize_and_align_labels(x, inputs_column="text", labels_column="summary"),
                batched=True,
                remove_columns=processed_datasets["whole"].column_names,
                load_from_cache_file=True
            )

            length = len(tokenized_datasets["whole"])
            train_size = int(0.9 * length)
            validate_size = length - train_size
            self.data_train, self.data_val = random_split(tokenized_datasets["whole"], [train_size, validate_size])


    def train_dataloader(self):
        return DataLoader(
            dataset=self.data_train,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_collator,
            shuffle=True,
        )

    def val_dataloader(self):
        return DataLoader(
            dataset=self.data_val,
            batch_size=self.hparams.train_batch_size,
            num_workers=self.hparams.num_workers,
            pin_memory=self.hparams.pin_memory,
            collate_fn=self.data_collator,
            shuffle=False,
        )
    #
    # def test_dataloader(self):
    #     return DataLoader(
    #         dataset=self.data_test,
    #         batch_size=self.hparams.train_batch_size,
    #         num_workers=self.hparams.num_workers,
    #         pin_memory=self.hparams.pin_memory,
    #         collate_fn=self.data_collator,
    #         shuffle=False,
    #     )

